@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -553,7 +553,7 @@ z_nf_dgram_socket_recv(gint fd, ZPktBuf **packet, ZSockAddr **from_addr, ZSockAd
             }
           else if (tos && cmsg->cmsg_level == SOL_IP && cmsg->cmsg_type == IP_TOS)
             {
-              *tos = *(int *) CMSG_DATA(cmsg);
+              memcpy(tos, CMSG_DATA(cmsg), sizeof(*tos));
             }
         }
 
