@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dispatch.c,v 1.40 2004/05/22 16:02:57 bazsi Exp $
- *
  * Author  : SaSa
  * Auditor :
  * Last audited version:
@@ -547,6 +545,10 @@ z_dispatch_chain_unref(ZDispatchChain *self)
   if (z_decref(&self->ref_cnt) == 0)
     {
       z_dispatch_chain_unlock(self);
+
+      if (self->accept_queue)
+        g_async_queue_unref(self->accept_queue);
+
       z_dispatch_bind_unref(self->registered_key);
       z_sockaddr_unref(self->bound_addr);
       g_free(self->session_id);
@@ -976,6 +978,7 @@ z_dispatch_bind_listener(ZDispatchChain *chain, ZDispatchBind **bound_key)
             {
               chain->listeners = g_list_remove(chain->listeners, entry);
               z_listener_entry_unref(entry);
+              z_listener_unref(listener);
               rc = FALSE;
               break;
             }
@@ -985,8 +988,8 @@ z_dispatch_bind_listener(ZDispatchChain *chain, ZDispatchBind **bound_key)
               chain->bound_addr = NULL;
               z_sockaddr_unref(listener->local);
               chain->listeners = g_list_remove(chain->listeners, entry);
-              z_listener_entry_unref(listener);
-
+              z_listener_entry_unref(entry);
+              z_listener_unref(listener);
               rc = FALSE;
               break;
 
