@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: proxy.c,v 1.132 2004/07/05 16:57:46 sasa Exp $
- *
  * Author  : Bazsi
  * Auditor : kisza
  * Last audited version: 1.23
@@ -36,6 +34,7 @@
 #include <zorp/streamfd.h>
 #include <zorp/streamline.h>
 #include <zorp/streambuf.h>
+#include <zorp/streamssl.h>
 #include <zorp/log.h>
 #include <zorp/io.h>
 #include <zorp/thread.h>
@@ -48,6 +47,8 @@
 #include <zorp/pyproxy.h>
 #include <zorp/pydispatch.h>
 #include <zorp/notification.h>
+#include <zorp/audit.h>
+#include <zorp/pyaudit.h>
 
 #include <stdarg.h>
 #include <sys/socket.h>
@@ -280,6 +281,10 @@ z_proxy_stop_request(const gchar *session_id)
   return verdict;
 }
 
+
+
+
+
 /**
  * z_proxy_hash_unref_proxy:
  * @key: not used
@@ -428,6 +433,8 @@ z_proxy_policy_config(ZProxy *self)
   
   z_proxy_set_state(self, ZPS_CONFIG);
 
+  z_policy_struct_set_is_config(self->ssl_opts.ssl_struct, TRUE);
+
   if (!z_proxy_policy_call(self, "config", NULL))
     {
       z_proxy_leave(self);
@@ -441,6 +448,8 @@ z_proxy_policy_config(ZProxy *self)
   z_policy_thread_release(self->thread);
 #endif
 
+  z_policy_struct_set_is_config(self->ssl_opts.ssl_struct, FALSE);
+
   z_proxy_leave(self);
   return TRUE;
 }
@@ -558,6 +567,47 @@ z_proxy_propagate_channel_props(ZProxy *self G_GNUC_UNUSED)
 }
 
 
+
+static gboolean
+z_proxy_set_server_address_no_acquire(ZProxy *self, const gchar *host, gint port)
+{
+  ZPolicyObj *res = NULL;
+  ZPolicyObj *args;
+  gint rc = FALSE;
+  gboolean called;
+
+  args = z_policy_var_build("(si)", host, port);
+  res = z_policy_call(self->handler, "setServerAddress", args, &called, self->session_id);
+  if (!res)
+    goto out;
+  if (!z_policy_var_parse(res, "i", &rc) || !rc)
+    goto out;
+
+  rc = TRUE;
+
+ out:
+  if (res)
+    z_policy_var_unref(res);
+  return rc;
+}
+
+/**
+ * Set server address for using by z_proxy_connect_server
+ * @param self[in,out]: ZProxy instance
+ * @param host[in]: remote server address
+ * @param port: remote server port
+ * @return: TRUE on success, FALSE otherwise
+ */
+gboolean
+z_proxy_set_server_address(ZProxy *self, const gchar *host, gint port)
+{
+  gint rc = FALSE;
+  z_policy_thread_acquire(self->thread);
+  rc = z_proxy_set_server_address_no_acquire(self, host, port);
+  z_policy_thread_release(self->thread);
+  return rc;
+}
+
 /**
  * z_proxy_connect_server:
  * @self: proxy instance
@@ -571,56 +621,73 @@ z_proxy_propagate_channel_props(ZProxy *self G_GNUC_UNUSED)
 gboolean
 z_proxy_connect_server(ZProxy *self, const gchar *host, gint port)
 {
-  ZPolicyObj *res, *args;
+  ZPolicyObj *res;
   gint rc;
   gboolean called;
   
   z_proxy_enter(self);
 
+  /* It might be possible that we already connected to the server: if
+     the SSL handshake order is server-client, we *do* have to connect
+     to the server before starting the client handshake. If this is
+     the case we simply return the already established connection. */
+  if (self->endpoints[EP_SERVER] &&
+      !z_stream_broken(self->endpoints[EP_SERVER]))
+    {
+      z_proxy_log(self, CORE_INFO, 6, "Using already established server connection;");
+      z_proxy_return(self, TRUE);
+    }
+
   z_proxy_propagate_channel_props(self);
 
-  z_policy_thread_acquire(self->thread);
-  self->endpoints[EP_SERVER] = NULL;
-  if (host && host[0])
+  if (self->endpoints[EP_SERVER])
     {
-      args = z_policy_var_build("(si)", host, port);
-      res = z_policy_call(self->handler, "setServerAddress", args, &called, self->session_id);
-      if (!res)
-        {
-          z_policy_thread_release(self->thread);
-          z_proxy_leave(self);
-          return FALSE;
-        }
-      if (!z_policy_var_parse(res, "i", &rc) || !rc)
-        {
-          z_policy_thread_release(self->thread);
-          z_proxy_leave(self);
-          return FALSE;
-        }
-      z_policy_var_unref(res);
+      z_stream_shutdown(self->endpoints[EP_SERVER], SHUT_RDWR, NULL);
+      z_stream_close(self->endpoints[EP_SERVER], NULL);
+      z_stream_unref(self->endpoints[EP_SERVER]);
+      self->endpoints[EP_SERVER] = NULL;
+
+      z_proxy_ssl_clear_session(self, EP_SERVER);
+    }
+
+  z_policy_thread_acquire(self->thread);
+  if (host && host[0] && !z_proxy_set_server_address_no_acquire(self, host, port))
+    {
+      z_policy_thread_release(self->thread);
+      z_proxy_return(self, FALSE);
     }
 
   res = z_policy_call(self->handler, "connectServer", NULL, &called, self->session_id);
 
-
   if (res && z_policy_stream_check(res))
     {
       self->endpoints[EP_SERVER] = z_policy_stream_get_stream(res);
-      rc = TRUE;
     }
   else
     {
       rc = FALSE;
+      goto error;
     }
 
- error:    
   z_policy_var_unref(res);
   z_policy_thread_release(self->thread);
 
   z_proxy_propagate_channel_props(self);
 
-  z_proxy_leave(self);
-  return rc;
+  if (self->endpoints[EP_SERVER])
+    rc = z_proxy_ssl_init_stream(self, EP_SERVER);
+  else
+    rc = FALSE;
+
+  z_proxy_return(self, rc);
+
+ error:
+  z_policy_var_unref(res);
+  z_policy_thread_release(self->thread);
+
+  z_proxy_propagate_channel_props(self);
+
+  z_proxy_return(self, FALSE);
 }
 
 /**
@@ -964,6 +1031,11 @@ z_proxy_var_register_va(ZProxy *s, ZPolicyDict *dict, const gchar *name, guint f
                              Z_VT_INT, name, flags, va_arg(args, gint *), NULL, 
                              NULL);
       break;
+    case Z_VAR_TYPE_INT64:
+      z_policy_dict_register(dict,
+                             Z_VT_INT64, name, flags, va_arg(args, gint *), NULL,
+                             NULL);
+      break;
     case Z_VAR_TYPE_STRING:
       z_policy_dict_register(dict, 
                              Z_VT_STRING, name, flags | Z_VF_CONSUME, va_arg(args, GString *), NULL, 
@@ -1117,6 +1189,9 @@ z_proxy_config_method(ZProxy *self)
   z_proxy_var_new(self, "server_stream",
                   Z_VAR_TYPE_CUSTOM | Z_VAR_GET,
                   NULL, z_proxy_query_stream, NULL, NULL);
+
+  z_proxy_ssl_register_vars(self);
+
   return z_proxy_policy_config(self);
 }
 
@@ -1229,6 +1304,8 @@ z_proxy_destroy_method(ZProxy *self)
       z_policy_thread_acquire(thread);
       self->thread = NULL;
 
+      z_proxy_ssl_free_vars(self);
+
       dict = self->dict;
       self->dict = NULL;
       z_policy_dict_unwrap(dict, self->handler);
@@ -1263,7 +1340,8 @@ z_proxy_run(ZProxy *self)
 {
   z_proxy_enter(self);
   if (z_proxy_config(self) &&
-      z_proxy_startup(self))
+      z_proxy_startup(self) &&
+      z_proxy_ssl_init_stream(self, EP_CLIENT))
     {
       z_proxy_propagate_channel_props(self);
       z_proxy_main(self);
@@ -1326,7 +1404,7 @@ z_proxy_nonblocking_start(ZProxy *self, ZProxyGroup *proxy_group)
   z_proxy_set_group(self, proxy_group);
   success = z_proxy_config(self) &&
             z_proxy_startup(self) &&
-            z_proxy_nonblocking_init(self, z_proxy_group_get_poll(proxy_group));
+            z_proxy_ssl_init_stream_nonblocking(self, EP_CLIENT);
   return success;
 }
 
@@ -1440,6 +1518,7 @@ z_proxy_new(ZClass *proxy_class, ZProxyParams *params)
   z_python_unlock();
 
   z_proxy_register(self);
+  z_proxy_ssl_config_defaults(self);
 
   z_proxy_add_child(params->parent, self);
   z_return(self);
@@ -1577,6 +1656,7 @@ z_proxy_basic_iface_set_var_method(ZProxyBasicIface *self G_GNUC_UNUSED, const g
   return FALSE;
 }
 
+
 /**
  * z_proxy_basic_iface_new:
  * @class: class description
@@ -1652,4 +1732,3 @@ ZClass ZProxyHostIface__class =
   &z_proxy_host_iface_funcs.super
 };
 
-
