@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: pycore.c,v 1.10 2004/07/22 07:51:10 bazsi Exp $
- *
  * Author  : Bazsi
  * Auditor : kisza
  * Last audited version: 1.22
@@ -261,7 +259,7 @@ z_py_szig_event(PyObject *self G_GNUC_UNUSED, PyObject *args)
         gchar *name;
         PyObject *dict;
         PyObject *key, *value;
-        gssize i;
+        Z_PYTHON_SIZE_TYPE i;
         
         if (!PyArg_Parse(value_repr, "(sO)", &name, &dict))
           z_return(NULL);
@@ -275,44 +273,60 @@ z_py_szig_event(PyObject *self G_GNUC_UNUSED, PyObject *args)
         i = 0;
         while (PyDict_Next(dict, &i, &key, &value)) 
           {
-            if (!PyString_Check(key) || !PyString_Check(value))
+            if (PyString_Check(key))
+              {
+                if (PyString_Check(value))
+                  {
+                    z_szig_value_add_prop(sv, PyString_AsString(key), z_szig_value_new_string(PyString_AsString(value)));
+                  }
+                else if (PyInt_Check(value))
+                  {
+                    z_szig_value_add_prop(sv, PyString_AsString(key), z_szig_value_new_long(PyInt_AsLong(value)));
+                  }
+                else
+                  {
+                    z_szig_value_free(sv, TRUE);
+                    PyErr_SetString(PyExc_ValueError, "Z_SZIG_TYPE_PROPS requires a string->string or string->int mapping");
+                    z_return(NULL);
+                  }
+              }
+            else
               {
                 z_szig_value_free(sv, TRUE);
-                PyErr_SetString(PyExc_ValueError, "Z_SZIG_TYPE_PROPS requires a string->string mapping");
+                PyErr_SetString(PyExc_ValueError, "Z_SZIG_TYPE_PROPS cannot handle not string keys");
                 z_return(NULL);
               }
-            z_szig_value_add_prop(sv, PyString_AsString(key), z_szig_value_new_string(PyString_AsString(value)));
           }
       }
     break;
     
-    case Z_SZIG_TYPE_SERVICE_PROPS:
+    case Z_SZIG_TYPE_CONNECTION_PROPS:
       {
         gchar *service;
         gint instance_id, sec_conn_id, related_id;
         PyObject *dict;
         PyObject *key, *value;
-        gssize i;
+        Z_PYTHON_SIZE_TYPE i;
         
         if (!PyArg_Parse(value_repr, "(siiiO)", &service, &instance_id, &sec_conn_id, &related_id, &dict))
           z_return(NULL);
         if (!PyDict_Check(dict))
           {
-            PyErr_SetString(PyExc_ValueError, "Z_SZIG_TYPE_SERVICE_PROPS requires a mapping as 5th argument");
+            PyErr_SetString(PyExc_ValueError, "Z_SZIG_TYPE_CONNECTION_PROPS requires a mapping as 5th argument");
             z_return(NULL);
           }
         
-        sv = z_szig_value_new_service_props(service, instance_id, sec_conn_id, related_id, NULL);
+        sv = z_szig_value_new_connection_props(service, instance_id, sec_conn_id, related_id, NULL);
         i = 0;
         while (PyDict_Next(dict, &i, &key, &value)) 
           {
             if (!PyString_Check(key) || !PyString_Check(value))
               {
                 z_szig_value_free(sv, TRUE);
-                PyErr_SetString(PyExc_ValueError, "Z_SZIG_TYPE_SERVICE_PROPS requires a string->string mapping");
+                PyErr_SetString(PyExc_ValueError, "Z_SZIG_TYPE_CONNECTION_PROPS requires a string->string mapping");
                 z_return(NULL);
               }
-            z_szig_value_add_service_prop(sv, PyString_AsString(key), PyString_AsString(value));
+            z_szig_value_add_connection_prop(sv, PyString_AsString(key), PyString_AsString(value));
           }
       }
     break;
@@ -330,7 +344,9 @@ z_py_szig_event(PyObject *self G_GNUC_UNUSED, PyObject *args)
 /**
  *  **/
 static PyObject *
-z_policy_notify_event(PyObject *self G_GNUC_UNUSED, PyObject *args)
+z_policy_notify_event(PyObject *self G_GNUC_UNUSED,
+                      PyObject *args G_GNUC_UNUSED
+                     )
 {
   Py_XINCREF(Py_None);
   return Py_None;
