@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id$
- *
  * Author  : Bazsi
  * Auditor : 
  * Last audited version:
@@ -110,6 +108,7 @@ struct _ZPolicyDictEntry
     guint8 int8_value;
     guint16 int16_value;
     guint32 int32_value;
+    guint64 int64_value;
     gsize cstring_buflen;
     ZPolicyObj *object_ref;
     struct
@@ -155,6 +154,8 @@ z_policy_dict_entry_free(ZPolicyDictEntry *e)
 {
   if (e->flags & Z_VF_CONSUME)
     e->type_funcs->free_fn(e);
+
+  g_free((gchar *) e->name);
   g_free(e);
 }
 
@@ -221,6 +222,10 @@ z_policy_dict_int_parse_args(ZPolicyDict *self G_GNUC_UNUSED, ZPolicyDictEntry *
           e->value = &e->ts.int32_value;
           e->ts.int32_value = va_arg(args, guint32);
           break;
+        case Z_VT_INT64:
+          e->value = &e->ts.int64_value;
+          e->ts.int64_value = va_arg(args, guint64);
+          break;
         default:
           g_assert_not_reached();
           break;
@@ -270,6 +275,11 @@ z_policy_dict_int_get_value(ZPolicyDict *self G_GNUC_UNUSED, ZPolicyDictEntry  *
       if (e->flags & Z_VF_INT_NET)
         value = ntohl(value);
       break;
+    case Z_VT_INT64:
+      value = *(guint64 *) e->value;
+      if (e->flags & Z_VF_INT_NET)
+        value = GUINT64_FROM_BE(value);
+      break;
     default:
       g_assert_not_reached();
       break;
@@ -320,6 +330,11 @@ z_policy_dict_int_set_value(ZPolicyDict *self G_GNUC_UNUSED, ZPolicyDictEntry *e
         value = htonl(value);
       *((guint32 *) e->value) = value;
       break;
+    case Z_VT_INT64:
+      if (e->flags & Z_VF_INT_NET)
+        value = GUINT64_TO_BE(value);
+      *((guint64 *) e->value) = value;
+      break;
     default:
       g_assert_not_reached();
       break;
@@ -1735,6 +1750,7 @@ z_policy_dict_set_value(ZPolicyDict *self, gboolean is_config, const gchar *name
       else
         {
           z_log(NULL, CORE_POLICY, 3, "Attribute cannot be written; config='%d', name='%s'", is_config, name);
+          return -1;
         }
     }
   return 1;
@@ -1796,6 +1812,7 @@ ZPolicyDictType z_policy_dict_types[] =
   [Z_VT_INT8]    = { z_policy_dict_int_parse_args, z_policy_dict_int_get_value, z_policy_dict_int_set_value, NULL },
   [Z_VT_INT16]   = { z_policy_dict_int_parse_args, z_policy_dict_int_get_value, z_policy_dict_int_set_value, NULL },
   [Z_VT_INT32]   = { z_policy_dict_int_parse_args, z_policy_dict_int_get_value, z_policy_dict_int_set_value, NULL },
+  [Z_VT_INT64]   = { z_policy_dict_int_parse_args, z_policy_dict_int_get_value, z_policy_dict_int_set_value, NULL },
   [Z_VT_STRING]  = { z_policy_dict_string_parse_args, z_policy_dict_string_get_value, z_policy_dict_string_set_value, z_policy_dict_string_free },
   [Z_VT_CSTRING] = { z_policy_dict_string_parse_args, z_policy_dict_string_get_value, z_policy_dict_string_set_value, z_policy_dict_string_free },
   [Z_VT_METHOD]  = { z_policy_dict_method_parse_args, z_policy_dict_method_get_value, NULL, z_policy_dict_method_free },
@@ -1818,7 +1835,7 @@ z_policy_dict_register_va(ZPolicyDict *self, ZVarType var_type, va_list args)
   g_assert((guint) var_type < sizeof(z_policy_dict_types) / sizeof(z_policy_dict_types[0]));
 
   e = g_new0(ZPolicyDictEntry, 1);
-  e->name = va_arg(args, gchar *);
+  e->name = g_strdup(va_arg(args, gchar *));
   e->flags = va_arg(args, guint);
   e->type = var_type;
   e->type_funcs = &z_policy_dict_types[var_type];
@@ -1831,7 +1848,7 @@ z_policy_dict_register_va(ZPolicyDict *self, ZVarType var_type, va_list args)
   va_end(args_copy);
   
   g_hash_table_insert(self->vars, (gchar *) e->name, e);
-  
+
 }
 
 /**
