@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: pydispatch.c,v 1.29 2004/07/02 10:03:33 bazsi Exp $
- *
  * Author  : Bazsi
  * Auditor : 
  * Last audited version: 
@@ -37,7 +35,7 @@
 #include <zorp/log.h>
 #include <zorp/pysockaddr.h>
 #include <zorp/pystream.h>
-
+#include <zorp/kzorp.h>
 
 /* ZPolicyDispatchBind */
 
@@ -196,7 +194,7 @@ z_policy_dispatch_bind_new_instance_iface_group(ZPolicyObj *self G_GNUC_UNUSED,
   gchar *keywords[] = { "group", "port", "family", "protocol", NULL };
   ZDispatchBind *bind;
   ZPolicyObj *res, *group_obj;
-  guint group;
+  guint group = 0;
   guint protocol = ZD_PROTO_AUTO, port = 0, family = AF_INET;
 
   if (!PyArg_ParseTupleAndKeywords(args, kw_args, "Oi|ii", keywords, &group_obj, &port, &family, &protocol))
@@ -642,7 +640,40 @@ static PyTypeObject z_policy_dispatch_type =
   Z_PYTYPE_TRAILER
 };
 
+/**
+ * z_policy_dispatch_get_kzorp_result
+ * @o unused
+ * @args Python arguments: fd
+ *
+ * Queries the KZorp results for the fd passed in through the
+ * getsockopt() interface of KZorp.
+ *
+ * Returns: A tuple consisting of the results: (client_zone_name,
+ * server_zone_name, dispatcher_name, service_name) or None if the
+ * lookup was not successful.
+ */
+static PyObject *
+z_policy_dispatch_get_kzorp_result(PyObject *o G_GNUC_UNUSED, PyObject *args)
+{
+  gint fd;
+  struct z_kzorp_lookup_result buf;
+  PyObject *ret;
 
+  if (!PyArg_ParseTuple(args, "i", &fd))
+    return NULL;
+
+  memset(&buf, 0, sizeof(buf));
+
+  if (!z_kzorp_get_lookup_result(fd, &buf)) {
+    Py_XINCREF(Py_None);
+    return Py_None;
+  }
+
+  ret = Py_BuildValue("(ssss)", &buf.czone_name, &buf.szone_name,
+                      &buf.dispatcher_name, &buf.service_name);
+
+  return ret;
+}
 
 PyMethodDef z_policy_dispatch_funcs[] =
 {
@@ -650,6 +681,7 @@ PyMethodDef z_policy_dispatch_funcs[] =
   { "DBSockAddr",  (PyCFunction) z_policy_dispatch_bind_new_instance_sa, METH_VARARGS | METH_KEYWORDS, NULL },
   { "DBIface",  (PyCFunction) z_policy_dispatch_bind_new_instance_iface, METH_VARARGS | METH_KEYWORDS, NULL },
   { "DBIfaceGroup",  (PyCFunction) z_policy_dispatch_bind_new_instance_iface_group, METH_VARARGS | METH_KEYWORDS, NULL },
+  { "getKZorpResult", (PyCFunction) z_policy_dispatch_get_kzorp_result, METH_VARARGS, NULL },
   { NULL,      NULL, 0, NULL }   /* sentinel*/
 };
 
