@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: szig.c,v 1.32 2004/07/02 10:03:33 bazsi Exp $
- *
  * Author  : SaSa
  * Auditor :
  * Last audited version:
@@ -122,6 +120,7 @@
 
 
 #define Z_SZIG_MAX_LINE 4096
+#define Z_SZIG_STATS_INTERVAL 5000      /**< interval at which samples will be taken for the average aggregator */
 
 /**
  * ZSzigEventCallback:
@@ -160,22 +159,6 @@ typedef struct _ZSzigQueueItem
   ZSzigValue *param;
 } ZSzigQueueItem;
 
-/**
- * ZSzigNode:
- *
- * A node in the result tree.
- **/
-struct _ZSzigNode
-{
-  gchar *name;
-  
-  ZSzigValue value;
-  gpointer agr_data;
-  GDestroyNotify agr_notify;
-  
-  GPtrArray *children;
-};
-
 /**
  * ZSzigConnection:
  *
@@ -387,7 +370,7 @@ z_szig_value_new_string(const gchar *val)
 }
 
 /**
- * z_szig_value_add_service_prop:
+ * z_szig_value_add_connection_prop:
  * @v: ZSzigValue instance
  * @name: property name
  * @value: property value
@@ -396,10 +379,10 @@ z_szig_value_new_string(const gchar *val)
  * instance.
  **/
 void
-z_szig_value_add_service_prop(ZSzigValue *v, const gchar *name, const gchar *value)
+z_szig_value_add_connection_prop(ZSzigValue *v, const gchar *name, const gchar *value)
 {
   z_enter();
-  g_assert(v->type == Z_SZIG_TYPE_SERVICE_PROPS);
+  g_assert(v->type == Z_SZIG_TYPE_CONNECTION_PROPS);
   
   if (v->u.service_props.string_count == Z_SZIG_MAX_PROPS)
     {
@@ -413,7 +396,7 @@ z_szig_value_add_service_prop(ZSzigValue *v, const gchar *name, const gchar *val
 }
 
 /** 
- * z_szig_value_new_service_props_va:
+ * z_szig_value_new_connection_props_va:
  * @service: service name
  * @instance_id: instance id (e.g. sequence number of this service)
  * @sec_conn_id: secondary connection id
@@ -425,19 +408,19 @@ z_szig_value_add_service_prop(ZSzigValue *v, const gchar *name, const gchar *val
  * properties is passed as a va_list.
  **/
 ZSzigValue *
-z_szig_value_new_service_props_va(const gchar *service, gint instance_id, gushort sec_conn_id, gushort related_id, const gchar *name, va_list l)
+z_szig_value_new_connection_props_va(const gchar *service, gint instance_id, gushort sec_conn_id, gushort related_id, const gchar *name, va_list l)
 {
   ZSzigValue *v = g_new0(ZSzigValue, 1);
   
   z_enter();
-  v->type = Z_SZIG_TYPE_SERVICE_PROPS;
+  v->type = Z_SZIG_TYPE_CONNECTION_PROPS;
   v->u.service_props.name = g_strdup(service);
   v->u.service_props.instance_id = instance_id;
   v->u.service_props.sec_conn_id = sec_conn_id;
   v->u.service_props.related_id = related_id;
   while (name)
     {
-      z_szig_value_add_service_prop(v, name, va_arg(l, gchar *));
+      z_szig_value_add_connection_prop(v, name, va_arg(l, gchar *));
       name = va_arg(l, gchar *);
     }
   z_return(v);
@@ -445,7 +428,7 @@ z_szig_value_new_service_props_va(const gchar *service, gint instance_id, gushor
 
 
 /** 
- * z_szig_value_new_service_props:
+ * z_szig_value_new_connection_props:
  * @service: service name
  * @instance_id: instance id (e.g. sequence number of this service)
  * @sec_conn_id: secondary connection id
@@ -457,14 +440,14 @@ z_szig_value_new_service_props_va(const gchar *service, gint instance_id, gushor
  * properties are passed as variable arguments.
  **/
 ZSzigValue *
-z_szig_value_new_service_props(const gchar *service, gint instance_id, gushort sec_conn_id, gushort related_id, const gchar *name, ...)
+z_szig_value_new_connection_props(const gchar *service, gint instance_id, gushort sec_conn_id, gushort related_id, const gchar *name, ...)
 {
   ZSzigValue *v;
   va_list l;
   
   z_enter();
   va_start(l, name);
-  v = z_szig_value_new_service_props_va(service, instance_id, sec_conn_id, related_id, name, l);
+  v = z_szig_value_new_connection_props_va(service, instance_id, sec_conn_id, related_id, name, l);
   va_end(l);
   z_return(v);
 }
@@ -570,7 +553,7 @@ z_szig_value_free(ZSzigValue *v, gboolean free_inst)
           g_string_free(v->u.string_value, TRUE);
           break;
           
-        case Z_SZIG_TYPE_SERVICE_PROPS:
+        case Z_SZIG_TYPE_CONNECTION_PROPS:
           for (i = 0; i < v->u.service_props.string_count * 2; i++)
             g_free(v->u.service_props.string_list[i]);
           g_free(v->u.service_props.name);
@@ -890,7 +873,7 @@ z_szig_escape_name(const gchar *name, gchar **buf)
  *
  * Returns: the SZIG node if found and NULL otherwise
  **/
-static ZSzigNode *
+ZSzigNode *
 z_szig_tree_lookup(const gchar *node_name, gboolean create, ZSzigNode **parent, gint *parent_ndx)
 {
   gchar **split;
@@ -1000,40 +983,246 @@ z_szig_agr_count_dec(ZSzigNode *node, ZSzigEvent ev G_GNUC_UNUSED, ZSzigValue *p
   z_return();
 }
 
+/**
+ * This aggregator function stores the maximum of the values.
+ *
+ * @param[in, out] target_node result node
+ * @param          ev event, not used
+ * @param          p event parameter, not used
+ * @param[in]      user_data source node name
+ *
+ * This aggregator function should be called for every change of the source value. It will
+ * track the highest value seen over its invocations.
+ *
+ * @note the maximum starts from 0, so it won't work with negative values.
+ **/
+static void
+z_szig_agr_maximum(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZSzigValue *p G_GNUC_UNUSED, gpointer user_data)
+{
+  const gchar * const source_node_name = (const gchar * const) user_data;
+  ZSzigNode *source_node;
+  glong current_max, value;
+
+  z_enter();
+
+  source_node = z_szig_tree_lookup(source_node_name, FALSE, NULL, NULL);
+  if (!source_node)
+    {
+      /*LOG
+        This message indicates an internal error, please contact your Zorp support for assistance.
+       */
+      z_log(NULL, CORE_ERROR, 3, "Invalid maximum aggregator, no source node; source_node='%s'", source_node_name);
+      z_return();
+    }
+
+  if (target_node->value.type != Z_SZIG_TYPE_LONG)
+    {
+      target_node->value.type = Z_SZIG_TYPE_LONG;
+      current_max = target_node->value.u.long_value = 0;
+    }
+  else
+    current_max = z_szig_value_as_long(&target_node->value);
+
+  value = z_szig_value_as_long(&source_node->value);
+
+  if (value > current_max)
+    {
+      target_node->value.type = Z_SZIG_TYPE_LONG;
+      target_node->value.u.long_value = value;
+    }
+
+  z_leave();
+}
+
+/**
+ * State for the maximum aggregator function.
+ **/
+typedef struct _ZSzigMaxDiffState
+{
+  glong last_value;
+  ZSzigNode *source_node;       /**< doesn't own this object */
+} ZSzigMaxDiffState;
+
+/**
+ * This aggregator function stores the maximal increment seen between its invocations.
+ *
+ * @param[in, out] target_node result node
+ * @param          ev event, not used
+ * @param          p event parameter, not used
+ * @param[in]      user_data source node name
+ *
+ * This aggregator function should be called over regular intervals. It will track the highest
+ * increment of the value of the source node seen between two calls. The source node should be
+ * a sequence number, e.g. incremented for each event.
+ *
+ * @note the maximum starts from 0, so it won't work with negative values.
+ **/
+void
+z_szig_agr_maximum_diff(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZSzigValue *p G_GNUC_UNUSED, gpointer user_data)
+{
+  const gchar *source_node_name = (const gchar *) user_data;
+  ZSzigMaxDiffState *max_state = NULL;
+  glong current_value;
+  glong diff = 0;
+
+  z_enter();
+
+  max_state = (ZSzigMaxDiffState *)z_szig_node_get_data(target_node);
+  if (!max_state)       /* initialize state */
+    {
+      max_state = g_new0(ZSzigMaxDiffState, 1);
+      max_state->source_node = z_szig_tree_lookup(source_node_name, FALSE, NULL, NULL);
+      z_szig_node_set_data(target_node, max_state, g_free);
+    }
+
+  if (!max_state->source_node)
+    {
+      /*LOG
+	This message indicates an internal error, please contact your Zorp support for assistance.
+       */
+      z_log(NULL, CORE_ERROR, 3, "Invalid maximum aggregator, no source node; source_node='%s'", source_node_name);
+      z_return();
+    }
+
+  current_value = z_szig_value_as_long(&max_state->source_node->value);
+  diff = (current_value - max_state->last_value) * 1000 / Z_SZIG_STATS_INTERVAL;
+  max_state->last_value = current_value;
+
+  if ((target_node->value.type != Z_SZIG_TYPE_LONG) || (target_node->value.u.long_value < diff))
+    {
+      target_node->value.type = Z_SZIG_TYPE_LONG;
+      target_node->value.u.long_value = diff;
+    }
+
+  z_return();
+}
+
+/**
+ * State for the average aggregator function.
+ **/
 typedef struct _ZSzigAvgState
 {
-  GTimeVal last_time;
   glong last_value;
   ZSzigNode *source_node;
+  GQueue *values;       /**< values received in the last interval */
+  glong sum;            /**< sum of values */
+  glong interval;       /**< interval to average over, in seconds */
 } ZSzigAvgState;
 
 /**
- * z_szig_agr_average:
- * @target_node: result node
- * @ev: event, not used
- * @p: event parameter, assumed to be a time
- * @user_data: source node name
+ * Type of records to store in ZSzigAvgState::values.
+ **/
+typedef struct _ZSzigAvgStateValue
+{
+  glong value;          /**< data point */
+  GTimeVal value_time;  /**< time at which it was collected */
+} ZSzigAvgStateValue;
+
+/**
+ * Free the ZSzigAvgState structure, including the queue.
+ *
+ * @todo FIXME: check function name against naming conventions
+ **/
+static inline void
+z_szig_agr_average_free(gpointer data)
+{
+  ZSzigAvgState *self = (ZSzigAvgState *)data;
+
+  g_queue_free(self->values);
+  g_free(data);
+}
+
+/**
+ * Check if check_time is earlier (older) than (end_time - interval).
+ *
+ * @param[in] check_time time value to check
+ * @param[in] end_time end of interval the beginning of which is checked
+ * @param[in] interval length of interval in seconds
+ *
+ * @returns TRUE if check_time is earlier than the interval
+ *
+ * @todo FIXME: check function name against naming conventions
+ **/
+static inline gboolean
+z_szig_agr_average_is_older(GTimeVal check_time, GTimeVal end_time, glong interval)
+{
+  g_time_val_add(&end_time, -interval*1000000UL);  /* was passed by value -- end_time becomes beginning_time */
+  if (check_time.tv_sec < end_time.tv_sec)
+    return TRUE;
+  if (check_time.tv_sec > end_time.tv_sec)
+    return FALSE;
+  if (check_time.tv_usec < end_time.tv_usec)
+    return TRUE;
+  return FALSE;
+}
+
+/**
+ * Aggregator function to calculate the average value for a node over a specific
+ * time interval.
+ *
+ * @param[in, out] target_node result node
+ * @param          ev event, not used
+ * @param[in]      p event parameter, assumed to be a time
+ * @param[in]      user_data source node name
  * 
  * This aggregator function should be registered to an event with TIME
- * parameter, it calculates the average value for a node between the
- * previous and current invocations. The source node should be a sequence
- * number, e.g. incremented for each event.
+ * parameter, it calculates the average value for a node over a time interval
+ * that is currently specified by the name of the node: the last numeric characters
+ * of the name should be 1, 5 or 15 for 1 minute, 5 minutes and 15 minutes respectively.
+ * The source node should be a sequence number, e.g. incremented for each event.
+ * The increments will be averaged.
  **/ 
-void
-z_szig_agr_average(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZSzigValue *p, gpointer user_data)
+static void
+z_szig_agr_average_rate(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZSzigValue *p, gpointer user_data)
 {
   const gchar *source_node_name = (const gchar *) user_data;
   ZSzigAvgState *avg_state;
   const GTimeVal *current_time;
-  glong time_diff, current_value;
+  glong current_value;
+  ZSzigAvgStateValue *oldest;
+  ZSzigAvgStateValue *new_value;
+  glong diff = 0;
   
   z_enter();
   target_node->value.type = Z_SZIG_TYPE_LONG;
   avg_state = (ZSzigAvgState *) z_szig_node_get_data(target_node);
-  if (!avg_state)
+  if (!avg_state)       /* initialize state */
     {
+      char *last_char;
+
       avg_state = g_new0(ZSzigAvgState, 1);
-      z_szig_node_set_data(target_node, avg_state, g_free);
+      avg_state->values = g_queue_new();
+
+      /* parse the interval from the node name (which ends with avg1, avg5 or avg15) */
+      /** @todo FIXME: this is a really ugly solution, better find a better one. */
+      last_char = strchr(target_node->name, '\0') - 1;
+      if (*last_char == '1')            /* avg1 */
+        {
+          avg_state->interval = 1 * 60;
+        }
+      else if (*last_char == '5')       /* avg5 or avg15 */
+        {
+          last_char--;
+          if (*last_char == '1')        /* avg15 */
+            {
+              avg_state->interval = 15 * 60;
+            }
+          else                          /* avg5 */
+            {
+              avg_state->interval = 5 * 60;
+            }
+        }
+      else
+        {
+          /*LOG
+            This message indicates an internal error, please contact your Zorp support for assistance.
+           */
+          z_log(NULL, CORE_ERROR, 3, "Failed to parse interval from node name; target_node.name='%s'", target_node->name);
+            /** @todo FIXME: discuss log message format */
+          g_assert_not_reached();
+        }
+
+      z_szig_node_set_data(target_node, avg_state, z_szig_agr_average_free);
     }
   if (!avg_state->source_node)
     avg_state->source_node = z_szig_tree_lookup(source_node_name, FALSE, NULL, NULL);
@@ -1044,25 +1233,38 @@ z_szig_agr_average(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZSzigVal
 	This message indicates an internal error, please contact your Zorp support for assistance.
        */
       z_log(NULL, CORE_ERROR, 3, "Invalid average aggregator, no source node; source_node='%s'", source_node_name);
-      goto exit_nostore;
+      z_return();
     }
   current_time = z_szig_value_as_time(p);
   current_value = z_szig_value_as_long(&avg_state->source_node->value);
-  if (avg_state->last_time.tv_sec == 0 && avg_state->last_time.tv_usec == 0)
-    goto exit_store_current;
-  
-  target_node->value.type = Z_SZIG_TYPE_LONG;
-  time_diff = ((current_time->tv_sec - avg_state->last_time.tv_sec) * 1000000 + 
-              (current_time->tv_usec - avg_state->last_time.tv_usec)) / 1000000;
-  if (time_diff == 0)
-    time_diff = 1;
-  target_node->value.u.long_value = (current_value - avg_state->last_value) / time_diff;
-  
- exit_store_current:
-  avg_state->last_time = *current_time;
+  diff = current_value - avg_state->last_value;
   avg_state->last_value = current_value;
-  
- exit_nostore:
+
+  /* while first value in the queue (peek) exists and is too old */
+  oldest = g_queue_peek_head(avg_state->values);
+  while (oldest && z_szig_agr_average_is_older(oldest->value_time, *current_time, avg_state->interval))
+    {
+      /* subtract it from sum and pop it off the queue */
+      avg_state->sum -= oldest->value;
+      g_free(g_queue_pop_head(avg_state->values));
+      /* get the next one */
+      oldest = g_queue_peek_head(avg_state->values);
+    }
+  /* if the queue is empty, set the sum to 0 (just to be sure) */
+  if (g_queue_is_empty(avg_state->values))
+    avg_state->sum = 0;
+  /* put (diff, current_time) into the queue if diff isn't 0 (0-s don't need to be tracked) */
+  if (diff != 0) {
+      new_value = g_new0(ZSzigAvgStateValue, 1);
+      new_value->value = diff;
+      new_value->value_time = *current_time;
+      g_queue_push_tail(avg_state->values, new_value);
+      avg_state->sum += diff;
+  }
+  /* calculate average using sum and interval requested */
+  target_node->value.type = Z_SZIG_TYPE_LONG;
+  target_node->value.u.long_value = avg_state->sum / avg_state->interval;
+
   z_return();
 }
 
@@ -1095,7 +1297,161 @@ z_szig_agr_append_string(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZS
 }
 
 /**
- * z_szig_agr_flat_service_props:
+ * Internal state to preserve between calls to z_szig_agr_per_zone_count_print_entry.
+ **/
+typedef struct _ZSzigAgrCountPrintState
+{
+  GString *printout;
+  gboolean first;
+} ZSzigAgrCountPrintState;
+
+/**
+ * Convert an entry in the hash table used by z_szig_agr_per_zonepair_count into a string and print it into the state.
+ *
+ * @param[in]      z zone which must be a char *
+ * @param[in]      v value which must be a gulong
+ * @param[in, out] s state which must be a ZSzigArgCountPrintState and start with an empty printout and first=TRUE
+ *
+ * Intended for use with g_hash_table_foreach.
+ **/
+static void
+z_szig_agr_per_zone_count_print_entry(gpointer z, gpointer v, gpointer s)
+{
+  ZSzigAgrCountPrintState *state = (ZSzigAgrCountPrintState *)s;
+  char *zone = (char *)z;
+  gulong *value = v;
+
+  if (state->first)
+    state->first = FALSE;
+  else
+    g_string_append(state->printout, ", ");
+
+  g_string_append_printf(state->printout, "%s(%ld)", zone, *value);
+}
+
+/**
+ * GDestroyNotify-compatible function to free a GHashTable (for when we don't want to care about reference counting it).
+ *
+ * @param[in] data pointer to GHashTable
+ **/
+static void
+z_hash_table_free(gpointer data)
+{
+  GHashTable *hashtable = (GHashTable *)data;
+
+  g_hash_table_destroy(hashtable);      /* free keys and values */
+  g_free(data);                         /* free the table itself */
+}
+
+/**
+ * Update per-zonepair connection count for the service.
+ *
+ * @param[in, out] service szig node of service
+ * @param[in, out] related szig node of current connection
+ *
+ * @note refactor things done for each side into separate function(s)?
+ **/
+void
+z_szig_agr_per_zone_count(ZSzigNode *service, ZSzigNode *related)
+{
+  ZSzigNode *server_zone_node;
+  ZSzigNode *client_zone_node;
+  char *inbound_zone_name = NULL;
+  char *outbound_zone_name = NULL;
+  ZSzigNode *inbound_zones_node;
+  ZSzigNode *outbound_zones_node;
+  GHashTable *inbound_hash;
+  GHashTable *outbound_hash;
+  gulong *inbound_counter;
+  gulong *outbound_counter;
+  GString *inbound_stats_string;
+  GString *outbound_stats_string;
+  ZSzigAgrCountPrintState inbound_print_state;
+  ZSzigAgrCountPrintState outbound_print_state;
+
+  /* check if we know both the client zone and the server zone */
+  client_zone_node = z_szig_node_lookup_child(related, "client_zone", NULL);
+  if (!client_zone_node)
+    return;
+  if (client_zone_node->value.type != Z_SZIG_TYPE_STRING)
+    return;
+
+  server_zone_node = z_szig_node_lookup_child(related, "server_zone", NULL);
+  if (!server_zone_node)
+    return;
+  if (server_zone_node->value.type != Z_SZIG_TYPE_STRING)
+    return;
+
+  /* get keys for the hash tables */
+  inbound_zone_name = server_zone_node->value.u.string_value->str;
+  outbound_zone_name = client_zone_node->value.u.string_value->str;
+
+  /* find or create stat nodes */
+  g_static_mutex_lock(&result_tree_structure_lock);
+  inbound_zones_node = z_szig_node_add_named_child(service, "inbound_zones");
+  inbound_zones_node->value.type = Z_SZIG_TYPE_STRING;
+  outbound_zones_node = z_szig_node_add_named_child(service, "outbound_zones");
+  outbound_zones_node->value.type = Z_SZIG_TYPE_STRING;
+  g_static_mutex_unlock(&result_tree_structure_lock);
+
+  /* get or create hash tables (state) */
+  inbound_hash = (GHashTable *)z_szig_node_get_data(inbound_zones_node);
+  if (!inbound_hash)
+    {
+      inbound_hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+      z_szig_node_set_data(inbound_zones_node, inbound_hash, z_hash_table_free);
+    }
+
+  outbound_hash = (GHashTable *)z_szig_node_get_data(outbound_zones_node);
+  if (!outbound_hash)
+    {
+      outbound_hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+      z_szig_node_set_data(outbound_zones_node, outbound_hash, z_hash_table_free);
+    }
+
+  /* lookup or create entry in hash tables */
+  inbound_counter = g_hash_table_lookup(inbound_hash, inbound_zone_name);
+  if (!inbound_counter)
+    {
+      inbound_counter = g_new0(gulong, 1);
+      g_hash_table_insert(inbound_hash, g_strdup(inbound_zone_name), inbound_counter);
+    }
+
+  outbound_counter = g_hash_table_lookup(outbound_hash, outbound_zone_name);
+  if (!outbound_counter)
+    {
+      outbound_counter = g_new0(gulong, 1);
+      g_hash_table_insert(outbound_hash, g_strdup(outbound_zone_name), outbound_counter);
+    }
+
+  /* increment counters */
+  (*inbound_counter)++;
+  (*outbound_counter)++;
+
+  /* update stats strings (node values) */
+  inbound_stats_string = g_string_sized_new(32); /* perhaps worth finetuning */
+  inbound_print_state.printout = inbound_stats_string;
+  inbound_print_state.first = TRUE;
+  g_hash_table_foreach(inbound_hash, z_szig_agr_per_zone_count_print_entry, &inbound_print_state);
+  g_static_mutex_lock(&result_node_gstring_lock);
+  if (inbound_zones_node->value.u.string_value)
+    g_string_free(inbound_zones_node->value.u.string_value, TRUE);
+  inbound_zones_node->value.u.string_value = inbound_print_state.printout;
+  g_static_mutex_unlock(&result_node_gstring_lock);
+
+  outbound_stats_string = g_string_sized_new(32); /* perhaps worth finetuning */
+  outbound_print_state.printout = outbound_stats_string;
+  outbound_print_state.first = TRUE;
+  g_hash_table_foreach(outbound_hash, z_szig_agr_per_zone_count_print_entry, &outbound_print_state);
+  g_static_mutex_lock(&result_node_gstring_lock);
+  if (outbound_zones_node->value.u.string_value)
+    g_string_free(outbound_zones_node->value.u.string_value, TRUE);
+  outbound_zones_node->value.u.string_value = outbound_print_state.printout;
+  g_static_mutex_unlock(&result_node_gstring_lock);
+}
+
+/**
+ * z_szig_agr_flat_connection_props:
  * @target_node: result node
  * @ev: event, not used
  * @p: event parameter, should be a service_props value
@@ -1105,7 +1461,7 @@ z_szig_agr_append_string(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZS
  * under target_node. It is used to represent service information.
  **/ 
 void
-z_szig_agr_flat_service_props(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZSzigValue *p, gpointer user_data G_GNUC_UNUSED)
+z_szig_agr_flat_connection_props(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZSzigValue *p, gpointer user_data G_GNUC_UNUSED)
 {
   ZSzigServiceProps *props;
   ZSzigNode *service, *instance, *sec_conn, *related, *node;
@@ -1113,7 +1469,7 @@ z_szig_agr_flat_service_props(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSE
   gint i;
   
   z_enter();
-  g_return_if_fail(p->type == Z_SZIG_TYPE_SERVICE_PROPS);
+  g_return_if_fail(p->type == Z_SZIG_TYPE_CONNECTION_PROPS);
   props = &p->u.service_props;
   
   /* create service node */
@@ -1136,11 +1492,15 @@ z_szig_agr_flat_service_props(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSE
       node->value.u.string_value = g_string_new(props->string_list[i * 2 + 1]);
     }
   g_static_mutex_unlock(&result_tree_structure_lock);
+
+  /* called here so that up-to-date data is already available in the tree */
+  z_szig_agr_per_zone_count(service, related);
+
   z_return();
 }
 
 /**
- * z_szig_agr_del_service_props:
+ * z_szig_agr_del_connection_props:
  * @target_node: result node
  * @ev: event, not used
  * @p: event parameter, should be a service_props value
@@ -1150,7 +1510,7 @@ z_szig_agr_flat_service_props(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSE
  * target_node.
  **/ 
 void
-z_szig_agr_del_service_props(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZSzigValue *p, gpointer user_data G_GNUC_UNUSED)
+z_szig_agr_del_connection_props(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZSzigValue *p, gpointer user_data G_GNUC_UNUSED)
 {
   ZSzigServiceProps *props;
   ZSzigNode *service, *instance;
@@ -1158,7 +1518,7 @@ z_szig_agr_del_service_props(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED
   gint ndx;
   
   z_enter();
-  g_return_if_fail(p->type == Z_SZIG_TYPE_SERVICE_PROPS);
+  g_return_if_fail(p->type == Z_SZIG_TYPE_CONNECTION_PROPS);
   props = &p->u.service_props;
   service = z_szig_node_lookup_child(target_node, props->name, NULL);
   g_snprintf(buf, sizeof(buf), "%d", props->instance_id);
@@ -1214,6 +1574,93 @@ z_szig_agr_flat_props(ZSzigNode *target_node, ZSzigEvent ev G_GNUC_UNUSED, ZSzig
   z_return();
 }
 
+/**
+ * This function aggregates the results of z_szig_agr_maximum_diff for all services.
+ **/
+void
+z_szig_agr_service_maximum_diff(ZSzigNode *target_node, ZSzigEvent ev, ZSzigValue *p, gpointer user_data)
+{
+  guint i;
+  z_enter();
+
+  for (i = 0; i < target_node->children->len; i++)
+    {
+      ZSzigNode *current_node = (ZSzigNode *) target_node->children->pdata[i];
+      ZSzigNode *max_node;
+      gchar *escaped_name = NULL;
+      gchar *max_nodename;
+      gchar *src_nodename;
+
+      escaped_name = z_szig_escape_name(current_node->name, &escaped_name);
+      max_nodename = g_strconcat("zorp.service.", escaped_name, ".", (gchar *)user_data, NULL);
+      src_nodename = g_strconcat("zorp.service.", escaped_name, ".session_number", NULL);
+      g_free(escaped_name);
+
+      max_node = z_szig_tree_lookup(max_nodename, TRUE, NULL, NULL);
+      z_szig_agr_maximum_diff(max_node, ev, p, (gpointer)src_nodename);
+
+      g_free(src_nodename);
+      g_free(max_nodename);
+    }
+  z_return();
+}
+
+void
+z_szig_agr_service_average_rate(ZSzigNode *target_node, ZSzigEvent ev, ZSzigValue *p, gpointer user_data)
+{
+  guint i;
+  z_enter();
+
+  for (i = 0; i < target_node->children->len; i++)
+    {
+      ZSzigNode *current_node = (ZSzigNode *) target_node->children->pdata[i];
+      ZSzigNode *avg_node;
+      gchar *avg_nodename;
+      gchar *src_nodename;
+      gchar *escaped_name = NULL;
+
+      escaped_name = z_szig_escape_name(current_node->name, &escaped_name);
+      avg_nodename = g_strconcat("zorp.service.", escaped_name, ".", (gchar *)user_data, NULL);
+      src_nodename = g_strconcat("zorp.service.", escaped_name, ".session_number", NULL);
+      g_free(escaped_name);
+
+      avg_node = z_szig_tree_lookup(avg_nodename, TRUE, NULL, NULL);
+      z_szig_agr_average_rate(avg_node, ev, p, (gpointer)src_nodename);
+
+      g_free(src_nodename);
+      g_free(avg_nodename);
+    }
+  z_return();
+}
+
+static void
+z_szig_agr_service_maximum(ZSzigNode *target_node, ZSzigEvent ev, ZSzigValue *p, gpointer user_data G_GNUC_UNUSED)
+{
+  guint i;
+
+  z_enter();
+
+  for (i = 0; i < target_node->children->len; i++)
+    {
+      const ZSzigNode * const current_node = (const ZSzigNode * const) target_node->children->pdata[i];
+      ZSzigNode *avg_node;
+      gchar *avg_nodename;
+      gchar *src_nodename;
+      gchar *escaped_name = NULL;
+
+      escaped_name = z_szig_escape_name(current_node->name, &escaped_name);
+      avg_nodename = g_strconcat("zorp.service.", escaped_name, ".sessions_max", NULL);
+      src_nodename = g_strconcat("zorp.service.", escaped_name, ".sessions_running", NULL);
+      g_free(escaped_name);
+
+      avg_node = z_szig_tree_lookup(avg_nodename, TRUE, NULL, NULL);
+      z_szig_agr_maximum(avg_node, ev, p, (gpointer)src_nodename);
+      g_free(src_nodename);
+      g_free(avg_nodename);
+    }
+
+  z_leave();
+}
 
 /* general framework */
 
@@ -1344,23 +1791,18 @@ z_szig_thread_stopped(ZThread *self G_GNUC_UNUSED, gpointer user_data G_GNUC_UNU
 }
 
 /**
- * z_szig_tick_callback:
- * @source: GSource instance
+ * This function is called every Z_SZIG_STATS_INTERVAL milliseconds to generate a SZIG event.
  *
- * This function is called every 1, 5, and 15 minutes to generate a SZIG event.
+ * @param[in] source GSource instance
  **/
 static gboolean
 z_szig_tick_callback(GSource *source)
 {
   GTimeVal current_time;
-  static gint ticks = 0;
+  static guint ticks = 0;
   
   g_source_get_current_time(source, &current_time);
-  z_szig_event(Z_SZIG_MIN1_TICK, z_szig_value_new_time(&current_time));
-  if ((ticks % 5) == 0)
-      z_szig_event(Z_SZIG_MIN5_TICK, z_szig_value_new_time(&current_time));
-  if ((ticks % 15) == 0)
-      z_szig_event(Z_SZIG_MIN15_TICK, z_szig_value_new_time(&current_time));    
+  z_szig_event(Z_SZIG_TICK, z_szig_value_new_time(&current_time));
 
   ticks++;
   return TRUE;
@@ -1713,16 +2155,27 @@ z_szig_init(const gchar *instance_name)
   memset(event_desc, 0, sizeof(event_desc));
   szig_queue = g_async_queue_new();
   
+  z_szig_register_handler(Z_SZIG_CONNECTION_START, z_szig_agr_count_inc, "zorp.stats.sessions_running", NULL);
+  z_szig_register_handler(Z_SZIG_CONNECTION_STOP, z_szig_agr_count_dec, "zorp.stats.sessions_running", NULL);
+
   z_szig_register_handler(Z_SZIG_THREAD_START, z_szig_agr_count_inc, "zorp.stats.threads_running", NULL);
   z_szig_register_handler(Z_SZIG_THREAD_STOP,  z_szig_agr_count_dec, "zorp.stats.threads_running", NULL);
   z_szig_register_handler(Z_SZIG_THREAD_START, z_szig_agr_count_inc, "zorp.stats.thread_number", NULL);
+  z_szig_register_handler(Z_SZIG_THREAD_START, z_szig_agr_maximum, "zorp.stats.threads_max", "zorp.stats.threads_running");
 
-  z_szig_register_handler(Z_SZIG_MIN1_TICK, z_szig_agr_average, "zorp.stats.thread_avg1", "zorp.stats.thread_number");
-  z_szig_register_handler(Z_SZIG_MIN5_TICK, z_szig_agr_average, "zorp.stats.thread_avg5", "zorp.stats.thread_number");
-  z_szig_register_handler(Z_SZIG_MIN15_TICK, z_szig_agr_average, "zorp.stats.thread_avg15", "zorp.stats.thread_number");
-  z_szig_register_handler(Z_SZIG_COUNTED_IP, z_szig_agr_append_string, "zorp.license.ipaddrs", NULL);
-  z_szig_register_handler(Z_SZIG_SERVICE_PROPS, z_szig_agr_flat_service_props, "zorp.conns", NULL);
-  z_szig_register_handler(Z_SZIG_SERVICE_STOP, z_szig_agr_del_service_props, "zorp.conns", NULL);
+  z_szig_register_handler(Z_SZIG_TICK, z_szig_agr_average_rate, "zorp.stats.thread_rate_avg1", "zorp.stats.thread_number");
+  z_szig_register_handler(Z_SZIG_TICK, z_szig_agr_average_rate, "zorp.stats.thread_rate_avg5", "zorp.stats.thread_number");
+  z_szig_register_handler(Z_SZIG_TICK, z_szig_agr_average_rate, "zorp.stats.thread_rate_avg15", "zorp.stats.thread_number");
+  z_szig_register_handler(Z_SZIG_TICK, z_szig_agr_maximum_diff, "zorp.stats.thread_rate_max", "zorp.stats.thread_number");
+  z_szig_register_handler(Z_SZIG_CONNECTION_PROPS, z_szig_agr_flat_connection_props, "zorp.conns", NULL);
+  z_szig_register_handler(Z_SZIG_CONNECTION_STOP, z_szig_agr_del_connection_props, "zorp.conns", NULL);
+
+  z_szig_register_handler(Z_SZIG_SERVICE_COUNT, z_szig_agr_flat_props, "zorp.service", NULL);
+  z_szig_register_handler(Z_SZIG_SERVICE_COUNT, z_szig_agr_service_maximum, "zorp.service", NULL);
+  z_szig_register_handler(Z_SZIG_TICK, z_szig_agr_service_average_rate, "zorp.service", "rate_avg1");
+  z_szig_register_handler(Z_SZIG_TICK, z_szig_agr_service_average_rate, "zorp.service", "rate_avg5");
+  z_szig_register_handler(Z_SZIG_TICK, z_szig_agr_service_average_rate, "zorp.service", "rate_avg15");
+  z_szig_register_handler(Z_SZIG_TICK, z_szig_agr_service_maximum_diff, "zorp.service", "rate_max");
 
   z_szig_register_handler(Z_SZIG_AUDIT_START, z_szig_agr_count_inc, "zorp.stats.audit_running", NULL);
   z_szig_register_handler(Z_SZIG_AUDIT_STOP,  z_szig_agr_count_dec, "zorp.stats.audit_running", NULL);
@@ -1731,9 +2184,13 @@ z_szig_init(const gchar *instance_name)
   z_szig_register_handler(Z_SZIG_RELOAD, z_szig_agr_flat_props, "zorp.info", NULL);
 
 
+  /* we need an offset of 2 to count the number of threads that were started before SZIG init */
+  z_szig_thread_started(NULL, NULL);
+  z_szig_thread_started(NULL, NULL);
+
   z_thread_register_start_callback((GFunc) z_szig_thread_started, NULL);
   z_thread_register_stop_callback((GFunc) z_szig_thread_stopped, NULL);
-  tick_source = g_timeout_source_new(60000);
+  tick_source = g_timeout_source_new(Z_SZIG_STATS_INTERVAL);
   g_source_set_callback(tick_source, (GSourceFunc) z_szig_tick_callback, tick_source, NULL);
   g_source_attach(tick_source, g_main_context_default());
 
