@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: policy.h,v 1.35 2004/08/05 14:14:24 sasa Exp $
- *
  ***************************************************************************/
 
 #ifndef ZORP_POLICY_H_INCLUDED
@@ -122,24 +120,14 @@ const gchar *z_verdict_str(ZVerdict verdict);
 #define z_policy_var_unref(v) do { Py_XDECREF(v); } while (0)
 #define z_policy_var_repr(v)  PyObject_Repr(v)
 
-gboolean z_policy_var_parse_str(PyObject *val, const gchar **result);
+gboolean z_policy_var_parse_str(PyObject *val, gchar **result);
 gboolean z_policy_var_parse_boolean(PyObject *val, gboolean *result);
 gboolean z_policy_var_parse_int(PyObject *val, gint *result);
+gboolean z_policy_var_parse_uint(PyObject *val, guint *result);
 gboolean z_policy_var_parse_int64(PyObject *val, gint64 *result);
+gboolean z_policy_var_parse_uint64(PyObject *val, guint64 *result);
 gboolean z_policy_var_parse_size(PyObject *val, gsize *result);
 
-static inline gboolean
-z_policy_var_parse_uint(PyObject *val, guint *result)
-{
-  return z_policy_var_parse_int(val, (gint *) result);
-}
-
-static inline gboolean
-z_policy_var_parse_uint64(PyObject *val, guint64 *result)
-{
-  return z_policy_var_parse_int64(val, (gint64 *) result);
-}
-
 static inline gboolean 
 z_policy_var_parse_ssize(PyObject *val, gssize *result)
 {
@@ -157,6 +145,7 @@ z_policy_var_parse_ssize(PyObject *val, gssize *result)
 #define z_policy_tuple_resize(s, n) PyTuple_Resize(s, n)
 #define z_policy_tuple_getitem(l, i) PyTuple_GetItem(l, i)
 #define z_policy_tuple_setitem(v, i, x) PyTuple_SetItem(v, i, x)
+#define z_policy_tuple_check(v) PyTuple_Check(v)
 
 #define z_policy_seq_check(v) PySequence_Check(v)
 #define z_policy_seq_getitem(v, i) PySequence_GetItem(v, i)
@@ -174,6 +163,7 @@ z_policy_var_parse_ssize(PyObject *val, gssize *result)
 #define z_policy_pdict_next(v, pos, k, vl) PyDict_Next(v, pos, k, vl)
 #define z_policy_pdict_new() PyDict_New()
 #define z_policy_pdict_getitem(v, i) PyDict_GetItem(v, i)
+#define z_policy_pdict_getitem_string(v, i) PyDict_GetItemString(v, i)
 #define z_policy_pdict_setitem(d, k, v) PyDict_SetItem(d, k, v)
 #define z_policy_pdict_setitem_string(d, k, v) PyDict_SetItemString(d, k, v)
 
