@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: proxy.h,v 1.82 2004/06/11 12:57:39 bazsi Exp $
- *
  ***************************************************************************/
 
 #ifndef ZORP_PROXY_H_INCLUDED
@@ -38,29 +36,8 @@
 #include <zorp/poll.h>
 #include <zorp/audit.h>
 #include <zorp/thread.h>
+#include <zorp/proxyssl.h>
 
-/* a two-way connection between streams */
-
-/* endpoint indexes */
-enum
-{
-  EP_CLIENT,
-  EP_SERVER,
-  EP_MAX
-};
-
-#define EP_OTHER(ep) (1-(ep))
-#define EP_STR(ep)   ((ep) == EP_CLIENT ? "client" : "server")
-
-enum
-{
-  EP_DIR_IN,
-  EP_DIR_OUT,
-  EP_DIR_MAX
-};
-
-#define EP_DIR_OTHER(ep) (1-(ep))
-#define EP_DIR_STR(ep)   ((ep) == EP_DIR_IN ? "input" : "output")
 
 /* proxy states, the order of these enums is important */
 enum
@@ -99,10 +76,10 @@ enum
 #define Z_VAR_TYPE_DIMHASH      0x00000800      /* variable is a multidimensional hash */
 #define Z_VAR_TYPE_ALIAS        0x00000900      /* variable is an alias of another variable */
 #define Z_VAR_TYPE_OBSOLETE     0x00000A00      /* variable is an obsolete alias of another variable */
+#define Z_VAR_TYPE_INT64        0x00000B00      /* variable is an int64 */
 
 
 typedef struct _ZProxyParams ZProxyParams;
-typedef struct _ZProxy ZProxy;
 typedef struct _ZProxyIface ZProxyIface;
 typedef struct _ZProxyFuncs ZProxyFuncs;
 typedef struct _ZProxyGroup ZProxyGroup;
@@ -138,11 +115,11 @@ struct _ZChannelProps
 struct _ZProxy
 {
   ZObject super;
-  guint16 status;
-  guint16 flags;
   gchar session_id[MAX_SESSION_ID];
   ZThread *proxy_thread;
   GThreadPriority proxy_pri;
+  guint16 status;
+  guint16 flags;
   ZPolicyThread *thread;
   ZPolicyObj *handler;
   ZPolicyDict *dict;
@@ -150,8 +127,6 @@ struct _ZProxy
   ZStream *endpoints[EP_MAX];
   ZPolicyObj *py_endpoints[EP_MAX];
 
-  ZChannelProps channel_props[EP_MAX];
-  gboolean channel_props_set[EP_MAX];
   GString *language;
 
   /* a pointer to the parent proxy */
@@ -161,7 +136,11 @@ struct _ZProxy
 
   GStaticMutex interfaces_lock;  
   GList *interfaces;
-  
+
+  ZProxySsl ssl_opts;
+
+  gboolean channel_props_set[EP_MAX];
+  ZChannelProps channel_props[EP_MAX];
 };
 
 extern ZClass ZProxy__class;
@@ -184,6 +163,13 @@ typedef ZProxy *(*ZProxyCreateFunc)(ZProxyParams *params);
     z_log_data_dump(((ZProxy *)self)->session_id, class, level, buf, len); \
   } while (0)
 
+#define z_proxy_pktbuf_data_dump(self, class, level, pktbuf)             \
+  do {									\
+    z_object_check_compatible((ZObject *) self, Z_CLASS(ZProxy));	\
+    /*NOLOG*/ 								\
+    z_pktbuf_data_dump(((ZProxy *)self)->session_id, class, level, pktbuf); \
+  } while (0)
+
 #define z_proxy_log_text_dump(self, class, level, buf, len)             \
   do {									\
     z_object_check_compatible((ZObject *) self, Z_CLASS(ZProxy));	\
@@ -226,6 +212,7 @@ ZProxyGroup *z_proxy_get_group(ZProxy *self);
 void z_proxy_set_group(ZProxy *self, ZProxyGroup *proxy_group);
 
 /* misc helper functions */
+gboolean z_proxy_set_server_address(ZProxy *self, const gchar *host, gint port);
 gint z_proxy_connect_server(ZProxy *self, const gchar *host, gint port);
 gint z_proxy_user_authenticated(ZProxy *self, const gchar *entity, gchar const **groups);
 
@@ -351,7 +338,12 @@ void z_proxy_iface_free_method(ZObject *s);
 
 /* ZProxyBasicIface */
 
-typedef ZProxyIface ZProxyBasicIface;
+/* NOTE: this interface is implemented by all proxies. dynamic references to
+ * self->parent should only be used through this interface as in this case it
+ * is ensured that the parent proxy exists by the time it is referenced.
+ */ 
+
+typedef ZProxyIface ZProxyBasicIface; 
 typedef struct _ZProxyBasicIfaceFuncs 
 {
   ZObjectFuncs super;
@@ -433,6 +425,8 @@ z_proxy_host_iface_check_name(ZProxyHostIface *s, const gchar *host_name, gchar
 }
 
 gboolean z_proxy_stop_request(const gchar *session_id);
+
+
 void z_proxy_hash_init(void);
 void z_proxy_hash_destroy(void);
 
