@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,13 +22,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: zorp.h,v 1.39 2004/07/05 07:59:55 bazsi Exp $
- *
  ***************************************************************************/
 
 #ifndef ZORP_H_INCLUDED
 #define ZORP_H_INCLUDED
 
+#ifdef _XOPEN_SOURCE
+#undef _XOPEN_SOURCE
+#endif
+
+#ifdef _POSIX_C_SOURCE
+#undef _POSIX_C_SOURCE
+#endif
+
 #include <Python.h>
 #undef ENABLE_IPV6
 #include <zorp/zorplibconfig.h>
@@ -49,8 +55,9 @@
 #define MAX_SESSION_ID		128
 #define DEADLOCK_CHECKER_DEFAULT_TIMEOUT  60
 
-#define CORE_POLICY  "core.policy"
-#define CORE_AUDIT   "core.audit"
+#define CORE_POLICY    "core.policy"
+#define CORE_AUDIT     "core.audit"
+#define CORE_VIOLATION "core.violation"
 
 #if SIZEOF_VOID_P == 4
 #   define G_GPOINTER_FORMAT "08" G_GSIZE_MODIFIER "x"
