@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: ftp.c,v 1.281 2004/07/23 09:36:59 sasa Exp $
- *
  * Author:  Andras Kis-Szabo <kisza@sch.bme.hu>
  * Author:  Attila SZALAY <sasa@balabit.hu>
  * Auditor:
@@ -43,6 +41,8 @@
 #include <zorp/thread.h>
 #include <zorp/io.h>
 #include <zorp/streamfd.h>
+#include <zorp/streamssl.h>
+#include <zorp/proxysslhostiface.h>
 #include <zorp/pystream.h>
 #include <zorp/pyproxy.h>
 #include <zorp/attach.h>
@@ -59,19 +59,16 @@
 GHashTable *ftp_command_hash = NULL;
 GHashTable *ftp_answer_hash  = NULL;
 
-
 void ftp_data_reset(FtpProxy *self);
 gboolean ftp_data_abort(FtpProxy *self);
 gboolean ftp_stream_write(FtpProxy *self,
                           char     side,
-                          guchar   *line,
+                          const guchar   *line,
                           guint    length);
 void ftp_proxy_free(ZObject *s);
 
 void ftp_command_reject(FtpProxy *self);
 
-static gboolean ftp_answer_write_with_setup(FtpProxy *self, gchar *answer_c, gchar *answer_p);
-
 gboolean ftp_answer_write(FtpProxy *self, gchar *msg);
 gboolean ftp_command_write_setup(FtpProxy *self, gchar *answer_c, gchar *answer_p);
 gboolean ftp_command_write(FtpProxy *self, char *msg);
@@ -122,7 +119,7 @@ ftp_connect_server_event(FtpProxy *self, gchar *hostname, guint port)
   z_proxy_return(self, TRUE);
 }
 
-static void
+void
 ftp_state_set(FtpProxy *self, guint order)
 {
   z_proxy_enter(self);
@@ -628,11 +625,45 @@ ftp_data_start(FtpProxy *self)
   z_proxy_return(self);
 }
 
+static gboolean
+ftp_data_do_ssl_handshake(FtpProxy *self, gint side)
+{
+  gboolean rc = TRUE;
+
+  z_proxy_enter(self);
+
+  /* require SSL if:
+   * - we're using FTPS and the protection level is private
+   * - the proxy requires SSL on this side
+   */
+  if ((self->data_protection_enabled[side])
+      || self->super.ssl_opts.security[side] == PROXY_SSL_SEC_FORCE_SSL)
+    {
+      ZProxySSLHandshake *handshake;
+      ZStream *old;
+
+      /* push an SSL stream on */
+      old = self->data_stream[side];
+      self->data_stream[side] = z_stream_ssl_new(old, NULL);
+      z_stream_unref(old);
+
+      /* do handshake */
+      handshake = z_proxy_ssl_handshake_new(&self->super, self->data_stream[side], side);
+
+      rc = z_proxy_ssl_perform_handshake(handshake);
+      if (!handshake->session)
+        rc = FALSE;
+    }
+
+  z_proxy_return(self, rc);
+}
+
 static gboolean
 ftp_data_create_transfer(FtpProxy *self)
 {
   gboolean rc = TRUE;
   ZStream *streams[EP_MAX];
+  gint first_side;
   gchar tmpsaddr_client[120];
   gchar tmpsaddr_server[120];
 
@@ -649,6 +680,22 @@ ftp_data_create_transfer(FtpProxy *self)
       z_proxy_return(self, FALSE);
     }
 
+  /* do SSL handshake if required */
+  if (self->super.ssl_opts.handshake_seq == PROXY_SSL_HS_CLIENT_SERVER)
+    first_side = EP_CLIENT;
+  else
+    first_side = EP_SERVER;
+
+  rc = ftp_data_do_ssl_handshake(self, first_side);
+  if (rc)
+    rc = ftp_data_do_ssl_handshake(self, EP_OTHER(first_side));
+
+  if (!rc)
+    {
+      z_proxy_log(self, FTP_ERROR, 3, "SSL handshake failed on data connection, cannot start transfer;");
+      z_proxy_return(self, FALSE);
+    }
+
   if (self->command_desc && self->command_desc->need_data == 2) /* data: cli -> svr */
     {
       streams[EP_CLIENT] = self->data_stream[EP_CLIENT];
@@ -859,7 +906,7 @@ ftp_data_abort(FtpProxy *self)
     {
       buf[0]=0xf2;
       rc = z_stream_write(self->super.endpoints[EP_SERVER], buf, 1, &len, NULL);
-      ftp_stream_write(self, 'S', "ABOR", 4);
+      ftp_stream_write(self, 'S', (const guchar *)"ABOR", 4);
     }
   z_proxy_return(self, rc == G_IO_STATUS_NORMAL);
 }
@@ -1044,13 +1091,37 @@ ftp_proxy_regvars(FtpProxy *self)
                   &self->data_mode);
 
   z_proxy_var_new(&self->super, "username",
-                  Z_VAR_TYPE_STRING | Z_VAR_GET,
+                  Z_VAR_TYPE_STRING | Z_VAR_GET | Z_VAR_SET,
                   self->username);
                   
   z_proxy_var_new(&self->super, "password",
-                  Z_VAR_TYPE_STRING | Z_VAR_GET,
+                  Z_VAR_TYPE_STRING | Z_VAR_GET | Z_VAR_SET,
                   self->password);
 
+  z_proxy_var_new(&self->super, "hostname",
+                  Z_VAR_TYPE_STRING | Z_VAR_GET | Z_VAR_SET,
+                  self->hostname);
+
+  z_proxy_var_new(&self->super, "hostport",
+                  Z_VAR_TYPE_INT | Z_VAR_GET | Z_VAR_SET,
+                  &self->hostport);
+
+  z_proxy_var_new(&self->super, "proxy_username",
+                  Z_VAR_TYPE_STRING | Z_VAR_GET | Z_VAR_SET,
+                  self->proxy_username);
+
+  z_proxy_var_new(&self->super, "proxy_password",
+                  Z_VAR_TYPE_STRING | Z_VAR_GET | Z_VAR_SET,
+                  self->proxy_password);
+
+  z_proxy_var_new(&self->super, "proxy_auth_needed",
+                  Z_VAR_TYPE_INT | Z_VAR_GET | Z_VAR_SET,
+                  &self->proxy_auth_needed);
+
+  z_proxy_var_new(&self->super, "auth",
+                  Z_VAR_TYPE_OBJECT | Z_VAR_GET | Z_VAR_SET_CONFIG,
+                  &self->auth);
+
   z_proxy_var_new(&self->super, "response_strip_msg",
                   Z_VAR_TYPE_INT | Z_VAR_GET | Z_VAR_SET_CONFIG,
                   &self->response_strip_msg);
@@ -1086,6 +1157,11 @@ ftp_proxy_regvars(FtpProxy *self)
   z_proxy_var_new(&self->super, "max_continuous_line",
                   Z_VAR_TYPE_INT | Z_VAR_GET | Z_VAR_SET_CONFIG,
                   &self->max_continuous_line);
+
+  z_proxy_var_new(&self->super, "features",
+                  Z_VAR_TYPE_HASH | Z_VAR_GET | Z_VAR_GET_CONFIG,
+                  self->policy_features);
+
   z_proxy_return(self);
 }
 
@@ -1109,6 +1185,10 @@ ftp_config_set_defaults(FtpProxy * self)
   self->username = g_string_new("");
   self->password = g_string_new("");
   self->hostname = g_string_new("");
+  self->proxy_username = g_string_new("");
+  self->proxy_password = g_string_new("");
+  self->proxy_auth_needed = 0;
+  self->auth_done = FALSE;
   
   self->data_mode = FTP_DATA_KEEP;
 
@@ -1120,6 +1200,7 @@ ftp_config_set_defaults(FtpProxy * self)
   self->max_continuous_line = 100;
   self->policy_command_hash = ftp_policy_command_hash_create();
   self->policy_answer_hash = ftp_policy_answer_hash_create();
+  self->policy_features = g_hash_table_new(g_str_hash, g_str_equal);
   self->request_cmd = g_string_sized_new(4);
   self->request_param = g_string_new("");
   self->answer_cmd = g_string_sized_new(4);
@@ -1129,6 +1210,10 @@ ftp_config_set_defaults(FtpProxy * self)
   self->poll = z_poll_new();
   self->data_port_min = 40000;
   self->data_port_max = 41000;
+  self->auth_tls_ok[EP_CLIENT] = FALSE;
+  self->auth_tls_ok[EP_SERVER] = FALSE;
+  self->data_protection_enabled[EP_CLIENT] = FALSE;
+  self->data_protection_enabled[EP_SERVER] = FALSE;
   self->valid_chars_username = g_string_new("a-zA-Z0-9._@");
   self->buffer_size = 4096;
   z_proxy_return(self);
@@ -1241,7 +1326,7 @@ ftp_read_line_get (FtpProxy * self, guint side, gint *error_value)
             break;
             
           case FTP_TELNET_IAC:
-            if (strchr (funcs, self->line[i]))
+            if (strchr ((const char *) funcs, self->line[i]))
               {
                 // in funcs
                 state = FTP_TELNET;
@@ -1250,7 +1335,7 @@ ftp_read_line_get (FtpProxy * self, guint side, gint *error_value)
                     i++;
                   }
               }
-            else if (strchr (negot, self->line[i]))
+            else if (strchr ((const char *) negot, self->line[i]))
               {
                 // in negotiation
                 state = FTP_TELNET_IAC_DW;
@@ -1287,7 +1372,7 @@ ftp_read_line_get (FtpProxy * self, guint side, gint *error_value)
 }
 
 gboolean
-ftp_stream_write(FtpProxy *self, char side, guchar *line, guint length)
+ftp_stream_write(FtpProxy *self, char side, const guchar *line, guint length)
 {
   gsize bytes_written = 0;
   gchar buf[2 * length + 3];
@@ -1404,6 +1489,9 @@ ftp_answer_fetch(FtpProxy *self, gboolean *continued)
     {
       z_cp();
       g_string_append_c(self->answer_param, '\n');
+
+      z_proxy_log(self, FTP_RESPONSE, 6, "Response continuation arrived; data='%s'", self->line);
+
       if(self->line_length < 4)
         {
           cont = TRUE;
@@ -1496,6 +1584,9 @@ ftp_answer_process(FtpProxy *self)
       ftp_answer_write_with_setup(self, self->answer_cmd->str, self->answer_param->str);
       break;
       
+    case FTP_NOOP:
+      break;
+
     default:
       self->state = FTP_QUIT;
       break;
@@ -1518,13 +1609,21 @@ ftp_answer_setup(FtpProxy *self, gchar *answer_c, gchar *answer_p)
     }
   else
     {
+      gboolean first_line = TRUE;
+
       while (tmp)
         {
           *tmp = 0;
-          g_string_append_printf(newline, "%s-%s\r\n", answer_c, answer_p);
+
+          if (first_line)
+            g_string_append_printf(newline, "%s-%s\r\n", answer_c, answer_p);
+          else
+            g_string_append_printf(newline, " %s\r\n", answer_p);
+
           *tmp = '\n';
           answer_p = tmp + 1;
           tmp = strchr(answer_p, '\n');
+          first_line = FALSE;
         }
         
       if (*answer_p)
@@ -1535,7 +1634,7 @@ ftp_answer_setup(FtpProxy *self, gchar *answer_c, gchar *answer_p)
   z_proxy_return(self, g_string_free(newline, FALSE));
 }
 
-static gboolean
+gboolean
 ftp_answer_write_with_setup(FtpProxy *self, gchar *answer_c, gchar *answer_p)
 {
   gchar *newline;
@@ -1563,7 +1662,7 @@ ftp_answer_write(FtpProxy *self, gchar *msg)
       else
         bytes_to_write = strlen(msg);
         
-      back = ftp_stream_write(self, 'C', msg, bytes_to_write);
+      back = ftp_stream_write(self, 'C', (const guchar *) msg, bytes_to_write);
     }
   self->drop_answer = FALSE;
   z_proxy_return(self, back);
@@ -1773,7 +1872,7 @@ ftp_command_write(FtpProxy *self, char *msg)
   gboolean back;
   
   z_proxy_enter(self);
-  back = ftp_stream_write(self, 'S', msg, bytes_to_write);
+  back = ftp_stream_write(self, 'S', (const guchar *) msg, bytes_to_write);
   z_proxy_return(self, back);
 }
 
@@ -1798,8 +1897,13 @@ void
 ftp_proto_nt_init(FtpProxy *self)
 {
   z_proxy_enter(self);
-  self->ftp_state = FTP_STATE_PRECONNECT;
-  SET_ANSWER(MSG_NON_TRANSPARENT_GREETING);
+
+  ftp_proto_state_set(self, FTP_STATE_PRECONNECT);
+
+  if (self->auth)
+    SET_ANSWER(MSG_NON_TRANSPARENT_GREETING_WITH_INBAND_AUTH)
+  else
+    SET_ANSWER(MSG_NON_TRANSPARENT_GREETING);
   ftp_answer_write_with_setup(self, self->answer_cmd->str, self->answer_param->str);
   self->state = FTP_NT_CLIENT_TO_PROXY;
   z_proxy_return(self);  
@@ -1825,11 +1929,19 @@ ftp_proto_nt_client_to_proxy(FtpProxy *self)
     case FTP_STATE_PRECONNECT_LOGIN_P:
       /* by this time login creditentials, and host name is received,
          try to connect to the remote server */
+
+      if (self->auth && !self->auth_done)
+        {
+          z_proxy_log(self, FTP_ERROR, 3, "Inband authentication is required but wasn't completed;");
+          self->state = FTP_QUIT;
+          break;
+        }
+
       if (ftp_connect_server_event(self, self->hostname->str, self->hostport) &&
           ftp_stream_server_init(self))
         {
           self->state = FTP_NT_SERVER_TO_PROXY;
-          self->ftp_state = FTP_STATE_PRECONNECT;
+          ftp_proto_state_set(self, FTP_STATE_PRECONNECT);
           g_string_assign(self->request_cmd, "");
         }
       else
@@ -1841,10 +1953,55 @@ ftp_proto_nt_client_to_proxy(FtpProxy *self)
     case FTP_STATE_PRECONNECT_QUIT:
       self->state = FTP_QUIT;
       break;
+
+    default:
+      /* nothing to do */
+      break;
     }
   z_proxy_return(self);  
 }
 
+static void
+ftp_proto_nt_send_USER(FtpProxy *self)
+{
+  gchar user_line[self->username->len + 6];
+
+  z_proxy_enter(self);
+
+  g_snprintf(user_line, sizeof(user_line), "USER %s", self->username->str);
+  g_string_assign(self->request_cmd, "USER");
+  ftp_command_write(self, user_line);
+  ftp_proto_state_set(self, FTP_STATE_PRECONNECT_LOGIN_U);
+
+  z_proxy_leave(self);
+}
+
+static gboolean
+ftp_nt_check_cert_subject(FtpProxy *self)
+{
+  ZProxyHostIface *host_iface;
+  gboolean res = TRUE;
+
+  z_proxy_enter(self);
+
+  host_iface = Z_CAST(z_proxy_find_iface(&self->super, Z_CLASS(ZProxyHostIface)), ZProxyHostIface);
+
+  if (host_iface)
+    {
+      gchar error_reason[256];
+
+      if (!z_proxy_host_iface_check_name(host_iface, self->hostname->str, error_reason, sizeof(error_reason)))
+        {
+          z_proxy_log(self, FTP_ERROR, 3, "Error checking hostname; error='%s'", error_reason);
+          res = FALSE;
+        }
+
+      z_object_unref(&host_iface->super);
+    }
+
+  z_proxy_return(self, res);
+}
+
 void
 ftp_proto_nt_server_to_proxy(FtpProxy *self)
 {
@@ -1882,15 +2039,114 @@ ftp_proto_nt_server_to_proxy(FtpProxy *self)
     case FTP_STATE_PRECONNECT:
       if (strcmp(self->answer_cmd->str, "220") == 0)
         {
-          gchar user_line[self->username->len + 6];
-
-          g_snprintf(user_line, sizeof(user_line), "USER %s", self->username->str);
-          g_string_assign(self->request_cmd, "USER");
-          ftp_command_write(self, user_line);
-          self->ftp_state = FTP_STATE_PRECONNECT_LOGIN_U;
+          if (self->auth_tls_ok[EP_CLIENT]
+              && (self->super.ssl_opts.security[EP_SERVER] == PROXY_SSL_SEC_FORWARD_STARTTLS))
+            {
+              /* send AUTH TLS */
+              g_string_assign(self->request_cmd, "AUTH");
+              ftp_command_write(self, "AUTH TLS");
+              ftp_proto_state_set(self, FTP_STATE_PRECONNECT_AUTH);
+            }
+          else
+            ftp_proto_nt_send_USER(self);
         }
       break;
       
+    case FTP_STATE_PRECONNECT_AUTH:
+      if (strcmp(self->answer_cmd->str, "234") == 0)
+        {
+          gboolean res;
+
+          /* AUTH TLS accepted by server, do handshake */
+          res = z_proxy_ssl_request_handshake(&self->super, EP_SERVER, TRUE);
+
+          if (!res)
+            {
+              z_proxy_log(self, FTP_ERROR, 2, "Server-side SSL handshake failed, terminating session;");
+              self->auth_tls_ok[EP_SERVER] = FALSE;
+
+              SET_ANSWER(MSG_NT_SERVER_HANDSHAKE_FAILED);
+              ftp_answer_write_with_setup(self, self->answer_cmd->str, self->answer_param->str);
+
+              self->state = FTP_QUIT;
+              break;
+            }
+          else
+            self->auth_tls_ok[EP_SERVER] = TRUE;
+
+          if (!ftp_nt_check_cert_subject(self))
+            {
+              z_proxy_log(self, FTP_ERROR, 2, "Server-side SSL certificate subject does not match inband hostname, terminating session;");
+
+              SET_ANSWER(MSG_NT_SERVER_CERT_INVALID_SUBJECT);
+              ftp_answer_write_with_setup(self, self->answer_cmd->str, self->answer_param->str);
+
+              self->state = FTP_QUIT;
+              break;
+            }
+
+          if (self->client_sent_pbsz)
+            {
+              g_string_assign(self->request_cmd, "PBSZ");
+              ftp_command_write(self, "PBSZ 0");
+              ftp_proto_state_set(self, FTP_STATE_PRECONNECT_PBSZ);
+            }
+          else
+            ftp_proto_nt_send_USER(self);
+        }
+      else
+        {
+          z_proxy_log(self, FTP_ERROR, 3, "Server did not accept AUTH TLS in non-transparent mode, aborting;");
+
+          SET_ANSWER(MSG_NT_SERVER_AUTH_REJECT);
+          ftp_answer_write_with_setup(self, self->answer_cmd->str, self->answer_param->str);
+
+          self->state = FTP_QUIT;
+          z_proxy_return(self);
+        }
+      break;
+
+    case FTP_STATE_PRECONNECT_PBSZ:
+      if (strcmp(self->answer_cmd->str, "200") == 0)
+        {
+          /* server accepted PBSZ 0: if PROT wasn't C, send prot;
+           * otherwise send USER */
+          if (self->data_protection_enabled[EP_CLIENT])
+            {
+              g_string_assign(self->request_cmd, "PROT");
+              ftp_command_write(self, "PROT P");
+              ftp_proto_state_set(self, FTP_STATE_PRECONNECT_PROT);
+            }
+          else
+            ftp_proto_nt_send_USER(self);
+        }
+      else
+        {
+          z_proxy_log(self, FTP_ERROR, 3, "Server did not accept PBSZ in non-transparent mode, aborting;");
+
+          SET_ANSWER(MSG_NT_SERVER_PBSZ_REJECT);
+          ftp_answer_write_with_setup(self, self->answer_cmd->str, self->answer_param->str);
+
+          self->state = FTP_QUIT;
+          z_proxy_return(self);
+        }
+      break;
+
+    case FTP_STATE_PRECONNECT_PROT:
+      if (strcmp(self->answer_cmd->str, "200") == 0)
+        ftp_proto_nt_send_USER(self);
+      else
+        {
+          z_proxy_log(self, FTP_ERROR, 3, "Server did not accept PROT in non-transparent mode, aborting;");
+
+          SET_ANSWER(MSG_NT_SERVER_PROT_REJECT);
+          ftp_answer_write_with_setup(self, self->answer_cmd->str, self->answer_param->str);
+
+          self->state = FTP_QUIT;
+          z_proxy_return(self);
+        }
+      break;
+
     case FTP_STATE_PRECONNECT_LOGIN_U:
       if (strcmp(self->answer_cmd->str, "331") == 0)
         {
@@ -1900,7 +2156,7 @@ ftp_proto_nt_server_to_proxy(FtpProxy *self)
           g_snprintf(pass_line, sizeof(pass_line), "PASS %s", self->password->str);
           g_string_assign(self->request_cmd, "PASS");
           ftp_command_write(self, pass_line);
-          self->ftp_state = FTP_STATE_LOGIN_P;
+          ftp_proto_state_set(self, FTP_STATE_LOGIN_P);
           self->state = FTP_SERVER_TO_CLIENT;
           ftp_state_set(self, EP_SERVER);
         }
@@ -1908,10 +2164,18 @@ ftp_proto_nt_server_to_proxy(FtpProxy *self)
         {
           /* no password required */
           ftp_answer_write(self, self->line);
-          self->ftp_state = FTP_STATE_CONVERSATION;
+          ftp_proto_state_set(self, FTP_STATE_CONVERSATION);
           ftp_state_set(self, EP_CLIENT);
           self->state = FTP_CLIENT_TO_SERVER;
         }
+      break;
+
+    default:
+      /* invalid state */
+      z_proxy_log(self, FTP_ERROR, 1, "Internal error while in non-transparent mode, proxy is in invalid state; state='%s'",
+                  ftp_proto_state_name(self->ftp_state));
+      self->state = FTP_QUIT;
+      break;
     }
   z_proxy_return(self);  
 }
@@ -2086,7 +2350,7 @@ ftp_main(ZProxy *s)
               {
                 self->state = FTP_SERVER_TO_CLIENT;
                 ftp_state_set(self, EP_SERVER);
-                self->ftp_state = FTP_STATE_LOGIN;
+                ftp_proto_state_set(self, FTP_STATE_LOGIN);
               }
             break;
 
@@ -2141,7 +2405,6 @@ ftp_proxy_free(ZObject *s)
   z_poll_quit(self->poll);
   z_poll_unref(self->poll);
   g_free(self->line);
-  g_string_free(self->hostname, TRUE);
   g_mutex_free(self->lock);
   if (self->preamble)
     g_free(self->preamble);
