@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: http.h,v 1.77 2004/06/12 19:54:37 bazsi Exp $
- *
  ***************************************************************************/
 
 #ifndef ZORP_MODULES_HTTP_H_INCLUDED
@@ -54,6 +52,7 @@
 #define HTTP_VIOLATION "http.violation"
 #define HTTP_ACCOUNTING "http.accounting"
 
+
 /* request specific constants */
 #define HTTP_REQ_ACCEPT       1
 #define HTTP_REQ_DENY         2
@@ -473,6 +472,10 @@ struct _HttpProxy
   /* Update the auth cache stamp at each request */
   gboolean auth_cache_update;
 
+
+  /* Categories the request falls into */
+  ZPolicyObj *request_categories;
+
   GString *append_cookie;
 };
 
@@ -553,6 +556,7 @@ extern GHashTable *response_proto_hash;
 extern GHashTable *request_hdr_proto_hash;
 extern GHashTable *response_hdr_proto_hash;
 
+
 static inline guint
 http_proto_lookup_hash(GHashTable *hash, const gchar *index)
 {
