@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: httpfltr.c,v 1.83 2004/07/26 11:45:57 bazsi Exp $
- *
  * Author: Balazs Scheidler <bazsi@balabit.hu>
  * Auditor: 
  * Last audited version: 
@@ -400,7 +398,7 @@ http_transfer_src_read(ZTransfer2 *s, ZStream *stream, gchar *buf, gsize count,
               /* for specified content-length, max_body_length has already
                  been processed, and content_length contains the number of
                  bytes to be transferred, but maximum max_body_length */                 
-              if (self->content_length == self->src_whole_length)
+              if (self->content_length >= 0 && (guint64) self->content_length == self->src_whole_length)
                 res = G_IO_STATUS_EOF;
               else
                 res = z_stream_read(stream, buf, MIN(count, self->content_length - self->src_whole_length), &br, &local_error);
@@ -804,7 +802,7 @@ http_transfer_stack_proxy(ZTransfer2 *s, ZStackedProxy **stacked)
     }
   
   if (stack_type != HTTP_STK_NONE)
-    success = z_proxy_stack_object(s->owner, stack_object, stacked);
+    success = z_proxy_stack_object(s->owner, stack_object, stacked, NULL);
  unref_unlock:
   z_policy_var_unref(proxy_stack_tuple);
   z_policy_unlock(self->super.owner->thread);
