@@ -76,16 +76,18 @@ struct
   gchar *fragment;
 } test_table[] =
 
+#define NA NULL, NULL, NULL, NULL, 0, NULL, NULL, NULL
+
 {
   { "http://user:pass@test.host:8080/file",    FALSE, FALSE, TRUE, "http", "user", "pass", "test.host", 8080, "/file",        NULL, NULL },
   { "http://user:pass@test.host/file",         FALSE, FALSE, TRUE, "http", "user", "pass", "test.host", 0,    "/file",        NULL, NULL },
   { "http://user:pass@test.host",              FALSE, FALSE, TRUE, "http", "user", "pass", "test.host", 0,    "/",            NULL, NULL },
-  { "http://user:pass@test.host?",             FALSE, FALSE, FALSE },
-  { "http://user:pass@test.host#",             FALSE, FALSE, FALSE },
+  { "http://user:pass@test.host?",             FALSE, FALSE, FALSE, NA },
+  { "http://user:pass@test.host#",             FALSE, FALSE, FALSE, NA },
   { "http://user:pass@test.host/file?query#fragment",         
                                                FALSE, FALSE, TRUE, "http", "user", "pass", "test.host", 0,    "/file",        "query", "fragment" },
   { "http://user:pass@test.host/file#fragment?start",
-                                               FALSE, FALSE, FALSE, },
+                                               FALSE, FALSE, FALSE, NA },
   { "http://user:pass@test.host/file#fragment",
                                                FALSE, FALSE, TRUE, "http", "user", "pass", "test.host", 0,    "/file",        NULL, "fragment" },
   { "http://user:pass@test.host/file?query",         
@@ -95,9 +97,9 @@ struct
   { "http://user@test.host/file",              FALSE, FALSE, TRUE, "http", "user", NULL,   "test.host", 0,    "/file",        NULL, NULL },
   { "http://test.host:8080/file",              FALSE, FALSE, TRUE, "http", NULL,   NULL,   "test.host", 8080, "/file",        NULL, NULL },
   { "http://test.host/file",                   FALSE, FALSE, TRUE, "http", NULL,   NULL,   "test.host", 0,    "/file",        NULL, NULL },
-  { "http://user:pass:test.host:54/file",      FALSE, FALSE, FALSE  },
+  { "http://user:pass:test.host:54/file",      FALSE, FALSE, FALSE, NA  },
   { "http://www.korridor.hu/default.ida?NNNN%u9090%u6858%ucbd3%u7801%u9090%u6858%ucbd3%u7801%u9090%u6858%ucbd3%u7801%u9090%u9090%u8190%u00c3%u0003%u8b00%u531b%u53ff%u0078%u0000%u0090=a", 
-                                               FALSE, FALSE, FALSE },
+                                               FALSE, FALSE, FALSE, NA },
   { "http://test.host/default.idaNNNN%u9090%u6858%ucbd3",
                                                FALSE, TRUE,  TRUE,  "http",  NULL,   NULL,  "test.host", 0,    "/default.ida\x4e\x4e\x4e\x4e\xe9\x82\x90\xe6\xa1\x98\xec\xaf\x93",
                                                                                                                               NULL, NULL },
@@ -110,26 +112,26 @@ struct
                                                FALSE, TRUE,  TRUE, "http", "user", "pass@", "test.host", 0,    "/fi/../le",        "%3Fa&%26", "%40" },
   /* invalid escaping, invalid_escape disabled */
   { "http://use%72:p%61ss%40@test.host/fi%2g%2e%2e%2fle?%u003f%61&%26#%40",  
-                                               FALSE, TRUE,  FALSE, },
+                                               FALSE, TRUE,  FALSE, NA },
   { "http://use%72:p%61ss%40@test.host/fi%2f%2e%2e%2fle?%u003g%61&%26#%40",  
-                                               FALSE, TRUE,  FALSE, },
+                                               FALSE, TRUE,  FALSE, NA },
   { "http://use%72:p%61ss%40@test.host/fi%2g%2e%2e%2fle?%u003f%61&%26#%40",  
-                                               FALSE, FALSE, FALSE, },
+                                               FALSE, FALSE, FALSE, NA },
   { "http://use%72:p%61ss%40@test.host/fi%2g%2e%2e%2fle?%u003f%61&%26#%40",  
-                                               FALSE, FALSE, FALSE, },
+                                               FALSE, FALSE, FALSE, NA },
   { "http://use%72:p%61ss%40@test.host/fi%2f%2e%2e%2fle?%u003f%61&%26#%4",  
-                                               FALSE, FALSE, FALSE, },
+                                               FALSE, FALSE, FALSE, NA },
   { "http://use%72:p%61ss%40@test.host/fi%2f%2e%2e%2fle?%u003f%61&%26#%u434",  
-                                               FALSE, TRUE,  FALSE, },
-  { "http//userpass@test.host/file",           FALSE, FALSE, FALSE },
-  { "http:userpass@test.host/file",            FALSE, FALSE, FALSE },
+                                               FALSE, TRUE,  FALSE, NA },
+  { "http//userpass@test.host/file",           FALSE, FALSE, FALSE, NA },
+  { "http:userpass@test.host/file",            FALSE, FALSE, FALSE, NA },
   { "http://user:pass@test.host/file?\x1b",    FALSE, FALSE, TRUE, "http", "user", "pass", "test.host", 0,    "/file",        "%1B", NULL },
 
   /* invalid escaping, invalid_escape, enabled */
   { "http://user:pass@test.host/f%2gile",      TRUE,  FALSE, TRUE, "http", "user", "pass", "test.host", 0,    "/f%2gile",        NULL, NULL },
   { "http://user:pass@test.host/f%u123gile",   TRUE,  TRUE,  TRUE, "http", "user", "pass", "test.host", 0,    "/f%u123gile",        NULL, NULL },
   
-  { NULL, }
+  { NULL, 0, 0, 0, NA }
 };
 
 int
