@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: telnetoption.c,v 1.10 2004/07/22 10:48:46 bazsi Exp $
- *
  * Author: Hidden
  * Auditor:
  * Last audited version:
@@ -66,7 +64,7 @@ telnet_opt_terminal_type(TelnetProxy *self, guint ep)
 {
   ZIOBuffer     *sbuf = &self->suboptions[ep];
   guint         ptr, i;
-  guchar        buf[SB_BUF_SIZE];
+  gchar         buf[SB_BUF_SIZE];
   guint         res;
 
   z_proxy_enter(self);
@@ -147,7 +145,7 @@ telnet_opt_terminal_speed(TelnetProxy *self, guint ep)
 {
   ZIOBuffer     *sbuf = &self->suboptions[ep];
   guint         ptr, i;
-  guchar        buf[SB_BUF_SIZE];
+  gchar         buf[SB_BUF_SIZE];
   guint         res;
 
   z_proxy_enter(self);
@@ -226,7 +224,7 @@ telnet_opt_x_display(TelnetProxy *self, guint ep)
 {
   ZIOBuffer     *sbuf = &self->suboptions[ep];
   guint         ptr, i;
-  guchar        buf[SB_BUF_SIZE];
+  gchar         buf[SB_BUF_SIZE];
   guint         res;
 
   z_proxy_enter(self);
@@ -242,9 +240,10 @@ telnet_opt_x_display(TelnetProxy *self, guint ep)
 
       for (ptr++; ptr < sbuf->end; ptr++)
         {
-          if (!isalnum(sbuf->buf[ptr]) && sbuf->buf[ptr] != '.' && sbuf->buf[ptr] != ':')
+          if (!isalnum(sbuf->buf[ptr]) && strchr(".:_-", sbuf->buf[ptr]) == NULL)
             {
-              z_proxy_log(self, TELNET_VIOLATION, 3, "X DISPLAY LOCATION IS option, invalid speed string;");
+              z_proxy_log(self, TELNET_VIOLATION, 3, "X DISPLAY LOCATION IS option, it contains invalid characters; value='%.*s'",
+                          (gint) (sbuf->end - (sbuf->ofs + 1)), &sbuf->buf[sbuf->ofs + 1]);
               z_proxy_return(self, TELNET_CHECK_ABORT);
             }
         }
@@ -309,7 +308,7 @@ telnet_opt_new_env(TelnetProxy *self, guint ep)
   ZIOBuffer     cbuf;
   guint         ptr, i;
   guint         res;
-  guchar        name[SB_BUF_SIZE], value[SB_BUF_SIZE];
+  gchar         name[SB_BUF_SIZE], value[SB_BUF_SIZE];
   guchar        command, type;
   gboolean      valid = FALSE; /* TRUE if there was anything accepted by policy */
 
@@ -553,7 +552,7 @@ telnet_opt_naws(TelnetProxy *self, guint ep)
   guint         ptr, i;
   guchar        buf[SB_BUF_SIZE];
   guint         res;
-  guchar        value[SB_BUF_SIZE];
+  gchar         value[SB_BUF_SIZE];
   guint16       width, height;
   gchar         width_cols[256], height_rows[256];
 
