@@ -1,7 +1,7 @@
 ############################################################################
 ##
 ## Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-## 2010 BalaBit IT Ltd, Budapest, Hungary
+## 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -164,7 +164,6 @@ from Cache import TimedCache, LockedCache
 
 import types, threading, time
 
-#####################################################
 ## Authentication
 #####################################################
 
@@ -421,7 +420,7 @@ class InbandAuthentication(AbstractAuthentication):
               InbandAuthentication class using the <parameter>auth</parameter> parameter.
             </para>
             <warning>
-              <para>Inband authentication is currently supported only for the Http and Ftp proxy classes. </para>
+              <para>Inband authentication is currently supported only for the Http, Ftp, and Socks proxy classes. </para>
             </warning>
           </description>
           <metainfo>
