@@ -1,7 +1,7 @@
 ############################################################################
 ##
 ## Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-## 2010 BalaBit IT Ltd, Budapest, Hungary
+## 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -290,8 +290,8 @@ Service(name="demo_service", proxy_class=HttpProxy, chainer=ConnectChainer(proto
 				log(session.session_id, CORE_SESSION, 3, 
 				    "Server connection established; server_fd='%d', server_address='%s', server_zone='%s', server_local='%s', server_protocol='%s'",
 				    (session.server_stream.fd, session.server_address, session.server_zone, session.server_local, session.protocol_name))
-				szigEvent(Z_SZIG_SERVICE_PROPS,
-				   (Z_SZIG_TYPE_SERVICE_PROPS,
+				szigEvent(Z_SZIG_CONNECTION_PROPS,
+				   (Z_SZIG_TYPE_CONNECTION_PROPS,
 				      (session.service.name, session.instance_id, 0, 0, {
 				        'server_address': str(session.server_address),
 				        'server_local': str(session.server_local),
@@ -300,7 +300,7 @@ Service(name="demo_service", proxy_class=HttpProxy, chainer=ConnectChainer(proto
 				 )))
 
 			return session.server_stream
-		raise DACException
+		raise DACException('Server connection is not permitted')
 		
 	def getNextTarget(self, session):
         	"""<method internal="yes">
