@@ -1,7 +1,7 @@
 ############################################################################
 ##
 ## Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-## 2010 BalaBit IT Ltd, Budapest, Hungary
+## 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -33,19 +33,20 @@
 """
   
 import new
+import socket
 
 from Zorp import *
 import Zorp
 from Zone import InetZone
 from Service import Service, PFService
-from SockAddr import SockAddrInet, SockAddrInetRange, SockAddrUnix
+from SockAddr import SockAddrInet, SockAddrInetHostname, SockAddrInetRange, SockAddrUnix
 
 
 from Router import TransparentRouter, DirectedRouter, InbandRouter
 from Chainer import ConnectChainer, MultiTargetChainer, StateBasedChainer, RoundRobinChainer, FailoverChainer, SideStackChainer
 from Domain import InetDomain
 from Listener import Listener, ZoneListener, CSZoneListener
-from Dispatch import Dispatcher, ZoneDispatcher, CSZoneDispatcher
+from Dispatch import Dispatcher, ZoneDispatcher, CSZoneDispatcher, NDimensionDispatcher
 from NAT import NATPolicy, ForgeClientSourceNAT, StaticNAT, OneToOneNAT, OneToOneMultiNAT, RandomNAT, HashNAT, GeneralNAT
 from Proxy import proxyLog
 from Auth import InbandAuthentication, AuthCache, AuthPolicy, AuthenticationPolicy
