@@ -1,7 +1,7 @@
 ############################################################################
 ##
 ## Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-## 2010 BalaBit IT Ltd, Budapest, Hungary
+## 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -106,12 +106,13 @@ from Zorp import *
 from Session import MasterSession
 from Cache import ShiftCache
 from Zone import root_zone
+from Domain import InetDomain
 
 from traceback import print_exc
 import Zorp, SockAddr
 import types, sys
 
-import kznf.kznfnetlink
+import kznf.kznfnetlink as kznf
 import socket
 
 listen_hook = None
@@ -536,9 +537,9 @@ Dispatcher(bindto=SockAddrInet('192.168.2.1', 50080), service="demo_http_service
                 # FIXME: handle multiple dispatch bind addresses
                 flags = 0
                 if self.transparent:
-                        flags = flags | kznf.kznfnetlink.KZF_DPT_TRANSPARENT
+                        flags = flags | kznf.KZF_DPT_TRANSPARENT
                 if self.__dict__.has_key("follow_parent") and self.follow_parent:
-                        flags = flags | kznf.kznfnetlink.KZF_DPT_FOLLOW_PARENT
+                        flags = flags | kznf.KZF_DPT_FOLLOW_PARENT
 
                 if isinstance(self.bindto, DBSockAddrType):
                         # SA bind
@@ -546,23 +547,23 @@ Dispatcher(bindto=SockAddrInet('192.168.2.1', 50080), service="demo_http_service
                                 rule_port = [(self.bindto.sa.port, self.bindto.sa.port)]
                         else:
                                 rule_port = self.rule_port
-                        messages.append((kznf.kznfnetlink.KZNL_MSG_ADD_DISPATCHER, kznf.kznfnetlink.create_add_dispatcher_sabind_msg(self.bindto.format(), flags, convertDBProtoToIPProto(self.bindto.protocol), self.bindto.sa.port, socket.ntohl(self.bindto.sa.ip), rule_port)))
+                        messages.append((kznf.KZNL_MSG_ADD_DISPATCHER, kznf.create_add_dispatcher_sabind_msg(self.bindto.format(), flags, convertDBProtoToIPProto(self.bindto.protocol), self.bindto.sa.port, socket.ntohl(self.bindto.sa.ip), rule_port)))
                 elif isinstance(self.bindto, DBIfaceType):
                         # interface bind
                         if self.rule_port == []:
                                 rule_port = [(self.bindto.port, self.bindto.port)]
                         else:
                                 rule_port = self.rule_port
-                        messages.append((kznf.kznfnetlink.KZNL_MSG_ADD_DISPATCHER, kznf.kznfnetlink.create_add_dispatcher_ifacebind_msg(self.bindto.format(), flags, convertDBProtoToIPProto(self.bindto.protocol), self.bindto.port, self.bindto.iface, rule_port, socket.ntohl(self.bindto.ip))))
+                        messages.append((kznf.KZNL_MSG_ADD_DISPATCHER, kznf.create_add_dispatcher_ifacebind_msg(self.bindto.format(), flags, convertDBProtoToIPProto(self.bindto.protocol), self.bindto.port, self.bindto.iface, rule_port, socket.ntohl(self.bindto.ip))))
                 elif isinstance(self.bindto, DBIfaceGroupType):
                         # interface group bind
                         if self.rule_port == []:
                                 rule_port = [(self.bindto.port, self.bindto.port)]
                         else:
                                 rule_port = self.rule_port
-                        messages.append((kznf.kznfnetlink.KZNL_MSG_ADD_DISPATCHER, kznf.kznfnetlink.create_add_dispatcher_ifgroupbind_msg(self.bindto.format(), flags, convertDBProtoToIPProto(self.bindto.protocol), self.bindto.port, self.bindto.group, 0xffffffff, rule_port)))
+                        messages.append((kznf.KZNL_MSG_ADD_DISPATCHER, kznf.create_add_dispatcher_ifgroupbind_msg(self.bindto.format(), flags, convertDBProtoToIPProto(self.bindto.protocol), self.bindto.port, self.bindto.group, 0xffffffff, rule_port)))
                 if self.service:
-                        messages.append((kznf.kznfnetlink.KZNL_MSG_ADD_DISPATCHER_CSS, kznf.kznfnetlink.create_add_dispatcher_css_msg(self.bindto.format(), self.service.name)))
+                        messages.append((kznf.KZNL_MSG_ADD_DISPATCHER_CSS, kznf.create_add_dispatcher_css_msg(self.bindto.format(), self.service.name)))
 
                 return messages
 
@@ -731,7 +732,7 @@ class ZoneDispatcher(Dispatcher):
 
                 # FIXME: check that the service exists
                 for zone in services.keys():
-                        messages.append((kznf.kznfnetlink.KZNL_MSG_ADD_DISPATCHER_CSS, kznf.kznfnetlink.create_add_dispatcher_css_msg(self.bindto.format(), self.services[zone], zone)))
+                        messages.append((kznf.KZNL_MSG_ADD_DISPATCHER_CSS, kznf.create_add_dispatcher_css_msg(self.bindto.format(), self.services[zone], zone)))
 
                 return messages
 
@@ -947,10 +948,211 @@ class CSZoneDispatcher(Dispatcher):
 
                 # FIXME: check that the service exists
                 for (c_zone, s_zone) in self.services.keys():
-                        messages.append((kznf.kznfnetlink.KZNL_MSG_ADD_DISPATCHER_CSS, kznf.kznfnetlink.create_add_dispatcher_css_msg(self.bindto.format(), self.services[(c_zone, s_zone)], c_zone, s_zone)))
+                        messages.append((kznf.KZNL_MSG_ADD_DISPATCHER_CSS, kznf.create_add_dispatcher_css_msg(self.bindto.format(), self.services[(c_zone, s_zone)], c_zone, s_zone)))
 
                 return messages
 
+class NDimensionDispatcher(Dispatcher):
+        """
+        <class maturity="stable">
+          <summary>Class encapsulating an N Dimension Dispatcher.</summary>
+          <description>
+          <example>
+                <title>NDimensionDispatcher example</title>
+                <para>The following example defines an NDimensionDispatcher that
+                starts the service called <parameter>internet_HTTP_DMZ</parameter>
+                for connections received on the <parameter>192.168.2.1</parameter>
+                 IP address and on the <parameter>eth0</parameter> interface,
+                 but only if the connection comes from the
+                 <parameter>internet</parameter> zone and the destination is
+                 in the <parameter>DMZ</parameter> zone.</para>
+                <synopsis>NDimensionDispatcher(bindto=DBSockAddr(SockAddrInet('0.0.0.0', 50010), ZD_PROTO_TCP), rules=( { 'iface': "eth0", 'proto': socket.IPPROTO_TCP, 'dst_port': "80", 'src_zone': 'internet', 'dst_zone': 'DMZ', 'service': 'internet_HTTP_DMZ' } ) )</synopsis>
+            </example>
+          </description>
+          <metainfo>
+            <attributes>
+              <attribute maturity="stable">
+                <name>services</name>
+                <type></type>
+                <description>services mapping indexed by zone names</description>
+              </attribute>
+            </attributes>
+          </metainfo>
+        </class>
+        """
+
+        def __init__(self, bindto=None, rules=None, **kw):
+                """
+                <method maturity="stable">
+                  <summary>Constructor to initialize a NDimensionDispatcher instance.</summary>
+                  <description>
+                    <para>
+                      This constructor initializes a NDimensionDispatcher instance and sets
+                      its initial attributes based on arguments.
+                    </para>
+                  </description>
+                  <metainfo>
+                    <arguments>
+                      <argument maturity="stable">
+                        <name>bindto</name>
+                        <type>
+                          <sockaddr existing="yes"/>
+                        </type>
+                       <description>An existing <link linkend="python.SockAddr">socket address</link> containing the IP address and port number where the Dispatcher accepts connections.</description>
+                       </argument>
+                    </arguments>
+                  </metainfo>
+                </method>
+                """
+                Dispatcher.__init__(self, bindto, None, **kw)
+                self.rules = rules
+
+        def getService(self, session):
+                """
+                <method internal="yes">
+                  <summary>Returns the service associated with the listener</summary>
+                  <description>
+                    <para>
+                      Returns the service to start.
+                    </para>
+                  </description>
+                  <metainfo>
+                    <arguments>
+                      <argument maturity="stable">
+                        <name>session</name>
+                        <type></type>
+                        <description>session reference</description>
+                      </argument>
+                    </arguments>
+                  </metainfo>
+                </method>
+                """
+                # query the KZorp result for the client fd of this session
+                fd = session.client_stream.fd
+                result = getKZorpResult(fd)
+                if (result):
+                        (client_zone_name, server_zone_name, dispatcher_name, service_name) = result
+
+                        return Globals.services.get(service_name)
+                else:
+                        ## LOG ##
+                        # This message indicates that the KZorp result
+                        # lookup has failed for this session.
+                        ##
+                        log(None, CORE_POLICY, 0, "Unable to determine service, KZorp service lookup failed; bindto='%s'", (self.bindto, ))
+                        return None
+
+        def _generateRuleId(self):
+                """<method internal="yes">
+                </method>
+                """
+                while (self._rule_id_index in self.rule_id_set):
+                        self._rule_id_index += 1
+
+                self.rule_id_set.add(self._rule_id_index)
+                return self._rule_id_index
+
+        def buildKZorpMessage(self):
+                """<method internal="yes">
+                </method>
+                """
+                flags = 0
+                if self.transparent:
+                        flags = flags | kznf.KZF_DPT_TRANSPARENT
+
+                messages = []
+                dimension_names = { 'auth' : kznf.KZA_N_DIMENSION_AUTH, 'iface' : kznf.KZA_N_DIMENSION_IFACE, 'ifgroup' : kznf.KZA_N_DIMENSION_IFGROUP, \
+                     'proto' : kznf.KZA_N_DIMENSION_PROTO, 'src_port' : kznf.KZA_N_DIMENSION_SRC_PORT, 'dst_port' : kznf.KZA_N_DIMENSION_DST_PORT, \
+                     'src_subnet' : kznf.KZA_N_DIMENSION_SRC_IP, 'src_zone' : kznf.KZA_N_DIMENSION_SRC_ZONE, 'dst_subnet' : kznf.KZA_N_DIMENSION_DST_IP, \
+                     'dst_zone' : kznf.KZA_N_DIMENSION_DST_ZONE }
+
+                self.rule_id_set = set()
+                self._rule_id_index = 1
+                if type(self.rules) != types.TupleType:
+                        self.rules = (self.rules, )
+
+                messages.append((kznf.KZNL_MSG_ADD_DISPATCHER, kznf.create_add_dispatcher_n_dimension(self.bindto.format(), flags, self.bindto.sa.port, len(self.rules))))
+
+                # check for duplicate rule IDs
+                for rule in self.rules:
+                        if 'rule_id' in rule:
+                                rule_id = rule['rule_id']
+                                if rule_id not in self.rule_id_set:
+                                        self.rule_id_set.add(rule_id)
+                                else:
+                                        raise ValueError, "Duplicated rule_id found; rule_id=%d" % (rule_id)
+
+                # generate rule IDs for rules not containing one
+                for rule in self.rules:
+                        if not 'rule_id' in rule:
+                                rule['rule_id'] = self._generateRuleId()
+
+                # sort self.rules so that rule IDs are in increasing order
+                if self.rules:
+                        rule_list = list(self.rules)
+                        rule_list.sort(lambda a, b: cmp(a['rule_id'], b['rule_id']))
+                        self.rules = rule_list
+
+                for rule in self.rules:
+                        entry_nums = {}
+
+                        if 'service' not in rule:
+                                raise ValueError, "Service key not found in rule description"
+
+                        service = rule['service']
+                        del rule['service']
+
+                        rule_id = rule['rule_id']
+                        del rule['rule_id']
+
+                        for k, v in rule.items():
+                                if k not in dimension_names:
+                                        raise ValueError, "Dimension name='%s' not found" % (k)
+
+                                nf_k = dimension_names[k]
+
+                                if k not in entry_nums:
+                                        entry_nums[nf_k] = 0
+
+                                if type(v) == types.TupleType:
+                                        entry_nums[nf_k] += len(v)
+                                elif k == 'src_port' or k == 'dst_port':
+                                        entry_nums[nf_k] += len(parsePortString(v))
+                                else:
+                                        entry_nums[nf_k] += 1
+
+                        messages.append((kznf.KZNL_MSG_ADD_RULE, kznf.create_add_n_dimension_rule_msg(self.bindto.format(), rule_id, service, entry_nums)))
+
+                        if (entry_nums.values() == []):
+                                _max = 0
+                        else:
+                                _max = max(entry_nums.values())
+
+                        for i in range(_max):
+                                data = {}
+
+                                for k, v in rule.items():
+                                        nf_k = dimension_names[k]
+
+                                        if nf_k in entry_nums and entry_nums[nf_k] > i:
+
+                                                if type(v) == types.TupleType:
+                                                        if k == 'iface' or k == 'ifgroup' or k == 'src_zone' or k =='dst_zone':
+                                                                data[nf_k] = v[i]
+                                                        elif k == 'src_subnet' or k == 'dst_subnet':
+                                                                addr = InetDomain(v[i])
+                                                                data[nf_k] = (socket.ntohl(addr.ip), socket.ntohl(addr.mask))
+
+                                                elif k == 'src_port' or k == 'dst_port':
+                                                        data[nf_k] = parsePortString(v)[i]
+                                                elif k == 'src_subnet' or k == 'dst_subnet':
+                                                        addr = InetDomain(v)
+                                                        data[nf_k] = (socket.ntohl(addr.ip), socket.ntohl(addr.mask))
+                                                else:
+                                                        data[nf_k] = v
+                                messages.append((kznf.KZNL_MSG_ADD_RULE_ENTRY, kznf.create_add_n_dimension_rule_entry_msg(self.bindto.format(), rule_id, data)))
+
+                return messages
 
 
 def purgeDispatches():
