@@ -1,7 +1,7 @@
 ############################################################################
 ##
 ## Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-## 2010 BalaBit IT Ltd, Budapest, Hungary
+## 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -582,6 +582,7 @@ class MyPsslProxy(PsslProxy):
                 """
                 self.instance_id = instance_id
                 self.session_id = "%s/%s:%d" % (self.base_session_id, self.name, self.instance_id)
+                self.master_session_id = self.session_id
 
 class StackedSession(AbstractSession):
         """
