@@ -1,7 +1,7 @@
 ############################################################################
 ##
 ## Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-## 2010 BalaBit IT Ltd, Budapest, Hungary
+## 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -127,11 +127,10 @@ InetZone('DMZ', ['10.50.0.0/32'],
 
 from Zorp import *
 from Domain import InetDomain, Inet6Domain
-from SockAddr import htonl
 from Cache import ShiftCache
+from socket import htonl, ntohl
 from traceback import print_exc
 import types
-import socket
 
 import kznf.kznfnetlink
 
@@ -879,8 +878,8 @@ class InetZone(Zone):
                         parent_name = None
 
                 if self.address:
-                        address = socket.ntohl(self.address.ip)
-                        mask = socket.ntohl(self.address.mask)
+                        address = ntohl(self.address.ip)
+                        mask = ntohl(self.address.mask)
                 else:
                         address = None
                         mask = None
