@@ -1,7 +1,7 @@
 ############################################################################
 ##
 ## Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-## 2010 BalaBit IT Ltd, Budapest, Hungary
+## 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -221,6 +221,7 @@ import Globals
 import Config
 import traceback
 import sys
+import errno
 
 config = Config
 
@@ -297,31 +298,100 @@ Z_SZIG_TYPE_LONG = 1
 Z_SZIG_TYPE_TIME = 2
 Z_SZIG_TYPE_STRING = 3
 Z_SZIG_TYPE_PROPS = 4
-Z_SZIG_TYPE_SERVICE_PROPS = 5
+Z_SZIG_TYPE_CONNECTION_PROPS = 5
 
-Z_SZIG_SERVICE_PROPS = 6
-Z_SZIG_SERVICE_STOP  = 7
+Z_SZIG_THREAD_START = 0
+Z_SZIG_THREAD_STOP = 1
+Z_SZIG_TICK = 2
+Z_SZIG_COUNTED_IP = 3
+Z_SZIG_CONNECTION_PROPS = 4
+Z_SZIG_CONNECTION_STOP = 5
+Z_SZIG_AUDIT_START = 6
+Z_SZIG_AUDIT_STOP = 7
+Z_SZIG_RELOAD = 8
+Z_SZIG_AUTH_PENDING_BEGIN = 9
+Z_SZIG_AUTH_PENDING_FINISH = 10
+Z_SZIG_SERVICE_COUNT = 11
+Z_SZIG_CONNECTION_START = 12
 
 Z_KEEPALIVE_NONE   = 0
 Z_KEEPALIVE_CLIENT = 1
 Z_KEEPALIVE_SERVER = 2
 Z_KEEPALIVE_BOTH   = 3
 
+Z_SSL_VERIFY_NONE		= 0
+Z_SSL_VERIFY_OPTIONAL_UNTRUSTED	= 1
+Z_SSL_VERIFY_OPTIONAL_TRUSTED	= 2
+Z_SSL_VERIFY_REQUIRED_UNTRUSTED	= 3
+Z_SSL_VERIFY_REQUIRED_TRUSTED	= 4
 
 from socket import AF_UNIX, AF_INET, SOCK_STREAM, SOCK_DGRAM
 
-ZoneException = "Zone not found"
-ServiceException = "Service"
-DACException = "DAC policy violation"
-MACException = "MAC policy violation"
-AAException = "Authentication or authorization failed"
+class ZorpException(Exception):
+    def __init__(self, detail):
+        self.what = ''
+        self.detail = detail
+
+    def __str__(self):
+        return '%s: %s' % (self.what, self.detail)
+
+class ZoneException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'Zone not found'
+
+class ServiceException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'Service'
+
+class DACException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'DAC policy violation'
+
+class MACException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'MAC policy violation'
+
+class AAException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'Authentication or authorization failed'
+
 # for compatibility
 AuthException = AAException
-LimitException = "Limit error"
-InternalException = "Internal error occured"
-UserException = "Incorrect, or unspecified parameter"
-LicenseException = "Attempt to use unlicensed components"
-MatcherException = "Matcher error"
+
+class LimitException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'Limit error'
+
+class InternalException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'Internal error occured'
+
+class UserException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'Incorrect, or unspecified parameter'
+
+class LicenseException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'Attempt to use unlicensed components'
+
+class MatcherException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'Matcher error'
+
+class ConfigException(ZorpException):
+    def __init__(self, detail):
+        ZorpException.__init__(self, detail)
+        self.what = 'Configuration error'
 
 def init(names):
         """
@@ -358,6 +428,51 @@ def init(names):
 		except IOError:
 			log(None, CORE_ERROR, 1, "Error reading audit encryption certificate; file='%s'", (config.audit.encrypt_certificate_file))
 
+        if config.audit.encrypt_certificate_list_file:
+                try:
+                        config.audit.encrypt_certificate_list = [ ]
+                        for list in config.audit.encrypt_certificate_list_file:
+                                newlist = [ ]
+                                for file in list:
+                                        try:
+                                                newlist.append( open(file, 'r').read() )
+                                        except IOError:
+                                                log(None, CORE_ERROR, 1, "Error reading audit encryption certificate; file='%s'", (file))
+                                config.audit.encrypt_certificate_list.append( newlist )
+                except TypeError:
+                        log(None, CORE_ERROR, 1, "Error iterating encryption certificate file list;")
+
+        if config.audit.encrypt_certificate_list == None and config.audit.encrypt_certificate:
+               config.audit.encrypt_certificate_list = [ [ config.audit.encrypt_certificate ] ]
+
+        if config.audit.sign_private_key_file:
+		try:
+			config.audit.sign_private_key = open(config.audit.sign_private_key_file, 'r').read()
+		except IOError:
+			log(None, CORE_ERROR, 1, "Error reading audit signature's private key; file='%s'", (config.audit.sign_private_key_file))
+
+        if config.audit.sign_certificate_file:
+		try:
+			config.audit.sign_certificate = open(config.audit.sign_certificate_file, 'r').read()
+		except IOError:
+			log(None, CORE_ERROR, 1, "Error reading audit signature's certificate; file='%s'", (config.audit.sign_certificate_file))
+
+        Globals.kzorp_responds_to_ping = False
+        if config.options.kzorp_enabled:
+                # ping kzorp to see if it's there
+                try:
+                        h = KZorp.openHandle()
+                        m = h.create_message(kznf.nfnetlink.NFNL_SUBSYS_KZORP, kznf.kznfnetlink.KZNL_MSG_GET_ZONE,
+                                             kznf.nfnetlink.NLM_F_REQUEST | kznf.nfnetlink.NLM_F_DUMP)
+                        m.set_nfmessage(kznf.kznfnetlink.create_get_zone_msg(None))
+                        result = h.talk(m, (0, 0), KZorp.netlinkmsg_handler)
+                        if result < 0:
+                                log(None, CORE_ERROR, 0, "Error pinging KZorp, it is probably unavailable; result='%d'" % (result))
+                        else:
+                                Globals.kzorp_responds_to_ping = True
+                except:
+                        log(None, CORE_ERROR, 0, "Error pinging KZorp, it is probably unavailable; exc_value='%s'" % (sys.exc_value))
+
         Globals.instance_name = names[0]
         for i in names:
                 try:
@@ -371,38 +486,37 @@ def init(names):
                         return FALSE
                 func()
 
-        # ping kzorp to see if it's there
-        kzorp_responds_to_ping = True
-        try:
-                h = KZorp.openHandle()
-                m = h.create_message(kznf.nfnetlink.NFNL_SUBSYS_KZORP, kznf.kznfnetlink.KZNL_MSG_GET_ZONE,
-                                     kznf.nfnetlink.NLM_F_REQUEST | kznf.nfnetlink.NLM_F_DUMP)
-                m.set_nfmessage(kznf.kznfnetlink.create_get_zone_msg(None))
-                result = h.talk(m, (0, 0), KZorp.netlinkmsg_handler)
-                if result == -errno.EINVAL:               # -22 was found empirically, this seems to be what it is
-                        log(None, CORE_ERROR, 0, "Error pinging KZorp, it is probably unavailable; result='%d'" % (result))
-                        kzorp_responds_to_ping = False
-        except:
-                log(None, CORE_ERROR, 0, "Error pinging KZorp, it is probably unavailable; exc_value='%s'" % (sys.exc_value))
-                kzorp_responds_to_ping = False
-
         if config.options.kzorp_enabled:
-                try:
-                        KZorp.downloadKZorpConfig(names[0])
-                except:
-                        ## LOG ##
-                        # This message indicates that downloading the necessary information to the
-                        # kernel-level KZorp subsystem has failed.
-                        ##
-                        log(None, CORE_ERROR, 0, "Error downloading KZorp configuration, Python traceback follows; error='%s'" % (sys.exc_value))
-                        for s in traceback.format_tb(sys.exc_traceback):
-                                for l in s.split("\n"):
-                                        if l:
-                                                log(None, CORE_ERROR, 0, "Traceback: %s" % (l))
+            # ping kzorp to see if it's there
+            try:
+                    h = KZorp.openHandle()
+                    m = h.create_message(kznf.nfnetlink.NFNL_SUBSYS_KZORP, kznf.kznfnetlink.KZNL_MSG_GET_ZONE,
+                                         kznf.nfnetlink.NLM_F_REQUEST)
+                    m.set_nfmessage(kznf.kznfnetlink.create_get_zone_msg("__nonexistent_zone__"))
+                    result = h.talk(m, (0, 0), KZorp.netlinkmsg_handler)
+                    if result < 0 and result != -errno.ENOENT:
+                            log(None, CORE_ERROR, 0, "Error pinging KZorp, it is probably unavailable; result='%d'" % (result))
+                    else:
+                            Globals.kzorp_responds_to_ping = True
+            except:
+                    log(None, CORE_ERROR, 0, "Error pinging KZorp, it is probably unavailable; exc_value='%s'" % (sys.exc_value))
 
-                        # if kzorp did respond to the ping, the configuration is erroneous -- we die here so the user finds out
-                        if kzorp_responds_to_ping:
-                                return FALSE
+            if Globals.kzorp_responds_to_ping:
+                    try:
+                            KZorp.downloadKZorpConfig(names[0])
+                    except:
+                            ## LOG ##
+                            # This message indicates that downloading the necessary information to the
+                            # kernel-level KZorp subsystem has failed.
+                            ##
+                            log(None, CORE_ERROR, 0, "Error downloading KZorp configuration, Python traceback follows; error='%s'" % (sys.exc_value))
+                            for s in traceback.format_tb(sys.exc_traceback):
+                                    for l in s.split("\n"):
+                                            if l:
+                                                    log(None, CORE_ERROR, 0, "Traceback: %s" % (l))
+
+                            # if kzorp did respond to the ping, the configuration is erroneous -- we die here so the user finds out
+                            return FALSE
 
         return TRUE
 
@@ -443,7 +557,7 @@ def cleanup(names):
 	##
 	log(None, CORE_DEBUG, 6, "Cleaning up instance; name='%s'", (names,))
 
-        if config.options.kzorp_enabled:
+        if Globals.kzorp_responds_to_ping and config.options.kzorp_enabled:
                 try:
                         KZorp.flushKZorpConfig(names[0])
                 except:
@@ -464,6 +578,8 @@ def notify(event, params):
         if Globals.notification_policy:
                 return Globals.notification_policy.notify(event, params)
 
+
+
 ## NOLOG ##
 		
 def log(sessionid, logclass, verbosity, msg, args=None):
