@@ -1,10 +1,11 @@
 from Zorp.Core import *
 from Zorp.Zorp import quit
 from Zorp.Zone import root_zone
-from Zorp.SockAddr import inet_ntoa, htonl
+from Zorp.SockAddr import inet_ntoa
 from Zorp.Session import MasterSession
 from traceback import *
 from time import time
+from socket import htonl
 
 config.options.kzorp_enabled = FALSE
 
