@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: main.c,v 1.27 2004/07/01 16:55:25 bazsi Exp $
- *
  * Author  : bazsi
  * Auditor : 
  * Last version : 
@@ -55,6 +53,8 @@
 #include <grp.h>
 #include <pwd.h>
 
+#include <zorp/misc.h>
+
 #define ZORP ZORP_LIBDIR    "/zorp"
 
 #if 1
@@ -65,6 +65,7 @@
 #define ZORP_ZORPCTL_CONF    "zorpctl.conf"
 #endif
 
+
 #define UNUSED __attribute__((unused))
 
 #define MAX_ZORPCTL_LINE_LENGTH 4096
@@ -221,7 +222,8 @@ z_get_uptime(void)
   f = fopen("/proc/uptime", "r");
   if (f)
     {
-      fscanf(f, "%lu", &i);
+      /* Fancy call to fscanf to avoid unused return value warning. */
+      if (fscanf(f, "%lu", &i)) { };
       fclose(f);
     }
   return i;
@@ -301,6 +303,37 @@ strduplen(char *line, int len)
   return res;
 }
 
+/**
+ * csvescape:
+ * @input: string to add escaping to
+ * @buf: result buffer
+ *
+ * Escapes ; and \ characters to provide a proper CSV output
+ * escaping various characters.
+ **/
+static gchar *
+csvescape(const gchar *input, gchar **buf)
+{
+  const guchar *src;
+  GString *dst;
+
+  dst = g_string_sized_new(32);
+  for (src = (guchar *)input; *src; src++)
+    {
+      if (*src == ';' || *src == '\\')
+        {
+          g_string_append_printf(dst, "\\%c", *src);
+        }
+      else
+        {
+          g_string_append_c(dst, *src);
+        }
+    }
+
+  *buf = dst->str;
+  return g_string_free(dst, FALSE);
+}
+
 static void
 z_sigalarm_handler(int signo UNUSED)
 {
@@ -337,8 +370,8 @@ z_error(int delay, const char *format, ...)
   va_start(args, format);
   if (delay)
     {
-      vsnprintf(buf, sizeof(buf), format, args);
-      strncat(delayed_errors, buf, sizeof(delayed_errors));
+      gsize len = vsnprintf(buf, sizeof(buf), format, args) + 1;
+      strncat(delayed_errors, buf, sizeof(delayed_errors) - strlen(delayed_errors) - len - 1);
     }
   else
     {
@@ -371,7 +404,12 @@ z_instance_running(ZInstance *inst, pid_t *pid, int *stale)
       return 0;
     }
   
-  fgets(buf, sizeof(buf), pidfile);
+  if (!fgets(buf, sizeof(buf), pidfile))
+    {
+      fclose(pidfile);
+      *stale = 1;
+      return 0;
+    }
   *pid = atoi(buf);
   fclose(pidfile);
   if (!(*pid))
@@ -511,7 +549,7 @@ z_parse_args(ZInstance *inst, char *zorp_args, char *zorpctl_args)
   char *buf = NULL;
   GOptionContext *ctx = NULL;
   
-  asprintf(&buf, "%s --as %s %s %s", ZORP, inst->name, zorp_args, SAFE_STR_EMPTY(zorp_append_args));
+  IGNORE_UNUSED_RESULT(asprintf(&buf, "%s --as %s %s %s", ZORP, inst->name, zorp_args, SAFE_STR_EMPTY(zorp_append_args)));
   if (!g_shell_parse_argv(buf,  &inst->zorp_argc, &inst->zorp_argv, NULL))
     {
       z_error(1, "Invalid zorp argument list: %s\n", zorp_args);
@@ -519,7 +557,7 @@ z_parse_args(ZInstance *inst, char *zorp_args, char *zorpctl_args)
     }
   g_free(buf);
 
-  asprintf(&buf, "zorpctl %s %s", SAFE_STR_EMPTY(zorpctl_args), SAFE_STR_EMPTY(zorpctl_append_args));
+  IGNORE_UNUSED_RESULT(asprintf(&buf, "zorpctl %s %s", SAFE_STR_EMPTY(zorpctl_args), SAFE_STR_EMPTY(zorpctl_append_args)));
   if (!g_shell_parse_argv(buf,  &inst->zorpctl_argc, &inst->zorpctl_argv, NULL))
     {
       z_error(1, "Invalid zorpctl argument list: %s\n", zorpctl_args);
@@ -545,7 +583,7 @@ z_parse_args(ZInstance *inst, char *zorp_args, char *zorpctl_args)
        * FIXME: this leaks memory.
        * */
       
-      asprintf(&buf, "%s --as %s %s %s", ZORP, inst->name, zorp_args, SAFE_STR_EMPTY(zorp_append_args));
+      IGNORE_UNUSED_RESULT(asprintf(&buf, "%s --as %s %s %s", ZORP, inst->name, zorp_args, SAFE_STR_EMPTY(zorp_append_args)));
       if (!g_shell_parse_argv(buf,  &inst->zorp_argc, &inst->zorp_argv, NULL))
         {
           z_error(1, "Invalid zorp argument list: %s\n", zorp_args);
@@ -776,10 +814,11 @@ z_parse_config(void)
   FILE *cfgf;
   char line[256], *value, *name, *eq;
   int lineno = 0;
+  int res = 1;
   
   cfgf = fopen(ZORP_ZORPCTL_CONF, "r");
   if (!cfgf)
-    return 1;
+    return 0;
     
   while (fgets(line, sizeof(line), cfgf))
     {
@@ -822,17 +861,20 @@ z_parse_config(void)
             z_parse_config_line_str("PIDFILE_DIR_MODE", name, value, &pidfile_dir_perm)))
         {
           z_error(0, "Unknown zorpctl.conf directive at %d: %s\n", lineno, line);
+          res = 0;
         }
     }
   if (process_limit_threshold != -1)
     {
       z_error(0, "The use of PROCESS_LIMIT_THRESHOLD in zorpctl.conf is deprecated, its value is ignored\n");
+      res = 0;
     }
   if (auto_restart_time_threshold != -1 || auto_restart_max_count != -1 || start_check_timeout != -1 || auto_restart_delay != -1.0)
     {
       z_error(0, "The use of AUTO_RESTART_DELAY, AUTO_RESTART_TIME_THRESHOLD, AUTO_RESTART_MAX_COUNT and START_CHECK_TIMEOUT in zorpctl.conf is deprecated, their values are ignored\n");
+      res = 0;
     }
-  return 1;
+  return res;
 }
 
 /**
@@ -916,13 +958,32 @@ z_process_kzorp_entries(ZInstance *ins, gpointer user_data G_GNUC_UNUSED)
 
   if ((file = fopen("/proc/net/nf_kzorp", "r")) == NULL)
     {
-      z_error(0, "Unable to open /proc/net/nf_kzorp. Kernel is too old or kzorp is not loaded.\n");
+      if (errno == ENOENT)
+        {
+          /* No such file exists: kzorp is most probably not available, so do
+           * not consider this an error. */
+          res = 1;
+        }
+      else
+        {
+          z_error(0, "Unable to open /proc/net/nf_kzorp. Kernel is too old or kzorp is not loaded.\n");
+        }
       goto error;
     }
 
-  while (!feof(file))
+  while (1)
     {
-      fgets(buffer, 4096, file);
+      if (!fgets(buffer, 4096, file) )
+        {
+          if (!feof(file))
+            {
+              z_error(0, "Error while reading from /proc/net/nf_kzorp.\n");
+              fclose(file);
+              goto error;
+            }
+          else
+            break;
+        }
 
       p = buffer;
 
@@ -991,7 +1052,6 @@ error:
   return res;
 }
 
-
 static int 
 z_get_counter(ZInstance *inst, const char *var_name)
 {
@@ -1035,7 +1095,6 @@ z_start_instance(ZInstance *inst)
   pid_t child;
   int status;
   int res = 1;
-  char fdlimbuf[16];
   
   child = fork();
   if (child == 0)
@@ -1267,67 +1326,129 @@ exit:
 }
 
 static void
-z_print_empty_result(char *status, pid_t pid, char *qry_strings[])
+z_szig_walk_details(ZSzigContext *ctx, const char *root, gboolean csv, const char *instance_name, int level)
 {
-  int i;
-  
-  printf("\"%s\";", status);
-  if (pid > 0)
-    printf("%d", pid);
-  for (i = 0; qry_strings[i]; i++)
+  char result[16384];
+  const char *root_basename = strrchr(root, '.');
+  gchar *escaped_name;
+
+  ++root_basename;
+
+  /* 0: <root>
+     1: <root>.ssh
+     2: <root>.ssh.1
+     3: <root>.ssh.1.0
+     4: <root>.ssh.1.0.0
+     5: <root>.ssh.1.0.0.param
+  */
+
+  if (!csv && level == 1)
+    printf("%s:\n", root_basename);
+  else if (!csv && level == 2)
+    printf(" %s:\n", root_basename);
+
+  z_szig_get_child(ctx, root, result, sizeof(result));
+  if (strcmp(result, "None") != 0)
     {
-      putc(';', stdout);
+      /* walk all children, deep first */
+      z_szig_walk_details(ctx, result, csv, instance_name, level + 1);
+
+      /* siblings */
+      z_szig_get_sibling(ctx, result, result, sizeof(result));
+      while (strcmp(result, "None") != 0)
+        {
+          z_szig_walk_details(ctx, result, csv, instance_name, level + 1);
+          z_szig_get_sibling(ctx, result, result, sizeof(result));
+        }
+    }
+
+  if (csv && level == 4)
+    {
+      printf("\n");
+    }
+  else if (level == 5)
+    {
+      z_szig_get_value(ctx, root, result, sizeof(result));
+      if (csv)
+        {
+          printf("%s=%s;", root_basename, csvescape(result, &escaped_name));
+          g_free(escaped_name);
+        }
+      else
+        printf("   %s: %s\n", root_basename, result);
     }
-  putc('\n', stdout);
 }
 
 static int
-z_process_gui_status_instance(ZInstance *inst, void *user_data UNUSED)
+z_process_authorize_list(ZInstance *inst, void *user_data)
+{
+  ZSzigContext *ctx;
+  gboolean csv = *(gboolean*)user_data;
+  
+  if (!csv)
+    printf("Instance %s:\n", inst->name);
+  ctx = z_szig_context_new(inst->name);
+  if (ctx)
+    z_szig_walk_details(ctx, "zorp.authorization.pending", csv, inst->name, 0);
+  else
+    goto szig_error;
+
+  goto exit;
+szig_error:
+  printf("error querying SZIG information\n");
+
+exit:
+  if (ctx)
+    z_szig_context_destroy(ctx);
+  return 1;
+}
+
+
+static int
+z_process_authorize(ZInstance *inst, void *user_data)
 {
   int stale;
   pid_t pid;
   ZSzigContext *ctx;
-  char *qry_strings[] = 
-  {
-    "zorp.stats.threads_running",
-    "zorp.stats.thread_number",
-    "zorp.stats.thread_avg1",
-    "zorp.stats.thread_avg5", 
-    "zorp.stats.thread_avg15",
-    NULL
-  };
-  int i;
-  
-  printf("\"%s\";", inst->name);
+  char result[16384];
+  gchar **params = (gchar **) user_data;
+
+  printf("Instance %s: ", inst->name);
   if (!z_instance_running(inst, &pid, &stale))
     {
       if (stale)
-        z_print_empty_result("missing", pid, qry_strings);
+        {
+          printf("stale pidfile, pid %d\n", pid);
+        }
       else
-        z_print_empty_result("missing", 0, qry_strings);
+        {
+          printf("not running\n");
+        }
       return 1;
     }
-
   ctx = z_szig_context_new(inst->name);
-  if (!ctx)
+  if (ctx)
     {
-      z_print_empty_result("noszig", pid, qry_strings);
-      return 1;
+      gboolean accept = params[0] != NULL;
+
+      z_szig_authorize(ctx, params[ accept ? 0 : 1 ], accept, params[2], result, sizeof(result));
+      printf("%s\n", result);
     }
-  printf("\"running\";\"%d\"", pid);
-  for (i = 0; qry_strings[i]; i++)
-    {
-      char result[256];
-      long val;
-      
-      z_szig_get_value(ctx, qry_strings[i], result, sizeof(result));
-      val = strtol(result, NULL, 10);
-      printf(";\"%ld\"", val);
-    }
-  printf("\n");
-  return 1;  
+  else
+    goto szig_error;
+
+  goto exit;
+szig_error:
+  printf("error querying SZIG information\n");
+
+exit:
+  if (ctx)
+    z_szig_context_destroy(ctx);
+  return 1;
 }
 
+
+
 static void
 z_szig_walk(ZSzigContext *ctx, const char *root)
 {
@@ -1377,6 +1498,28 @@ z_process_szig_walk_instance(ZInstance *inst, void *user_data)
   return 1;
 }
 
+static int
+z_process_conns_walk_instance(ZInstance *inst, void *user_data G_GNUC_UNUSED)
+{
+  ZSzigContext *ctx;
+  
+  ctx = z_szig_context_new(inst->name);
+  if (ctx)
+    z_szig_walk_details(ctx, "zorp.conns", TRUE, inst->name, 0);
+  else
+    goto szig_error;
+
+  goto exit;
+szig_error:
+  printf("error querying SZIG information\n");
+
+exit:
+  if (ctx)
+    z_szig_context_destroy(ctx);
+ 
+  return 1;
+}
+
 static int
 z_process_log_func(ZInstance *inst, char *cmd, char *param)
 {
@@ -1588,19 +1731,34 @@ z_process_coredump(ZInstance *inst, void *user_data UNUSED)
   return res;
 }
 
+static int
+z_process_args(char *ident, int argc, char *argv[], int (*func)(ZInstance *inst, void *user_data), void *user_data, int display_instances) G_GNUC_WARN_UNUSED_RESULT;
+
+static int
+z_process_args_withmsg( const char * msg, 
+char *ident, int argc, char *argv[], int (*func)(ZInstance *inst, void *user_data), void *user_data, int display_instances)
+{
+  int res;
+  printf("%s", msg);
+  res = z_process_args(ident, argc, argv, func, user_data, display_instances);
+  printf("\n");
+  return res;
+}
+
+
 static int
 z_process_args(char *ident, int argc, char *argv[], int (*func)(ZInstance *inst, void *user_data), void *user_data, int display_instances)
 {
   ZInstance *inst;
-  int success;
-  int res = 0;
+  int success_all = 1;
   int i;
 
   if (argc == 0)
     {
       for (inst = instances; inst; inst = inst->next)
         {
-          success = func(inst, user_data);
+          int success = func(inst, user_data);
+          success_all &= !!success;
           if (display_instances)
             printf("%s%s", inst->name, success ? " " : "! ");
         }
@@ -1618,16 +1776,18 @@ z_process_args(char *ident, int argc, char *argv[], int (*func)(ZInstance *inst,
             {
               while (fgets(inst_buf, sizeof(inst_buf), inst_list))
                 {
+                  int success = 0;
                   chomp(inst_buf);
                   inst = z_search_instance(inst_buf);
                   if (!inst)
                     {
                       z_error(1, "%s: No such instance: %s\n", ident, inst_buf);
-                      success = 0;
+                      success_all = 0;
                     }
                   else
                     {
                       success = func(inst, user_data);
+                      success_all &= !!success;
                     }
                   if (display_instances)
                     printf("%s%s", inst_buf, success ? " " : "! ");
@@ -1638,29 +1798,31 @@ z_process_args(char *ident, int argc, char *argv[], int (*func)(ZInstance *inst,
           else
             {
               z_error(1, "Error opening instance list file: %s.", &argv[0][1]);
+              success_all = 0;
             }
         }
       else
         {
           for (i = 0; i < argc; i++)
             {
+              int success = 0;
               inst = z_search_instance(argv[i]);
               if (!inst)
                 {
                   z_error(1, "%s: No such instance: %s\n", ident, argv[i]);
-                  success = 0;
+                  success_all = 0;
                 }
               else
                 {
                   success = func(inst, user_data);
+                  success_all &= !!success;
                 }
               if (display_instances)
                 printf("%s%s", argv[i], success ? " " : "! ");
             }
         }
     }
-  /* FIXME: return appropriate exit code */
-  return res;
+  return success_all ? 0 : 1;
 }
 
 /* command implementations */
@@ -1668,82 +1830,64 @@ z_process_args(char *ident, int argc, char *argv[], int (*func)(ZInstance *inst,
 static int
 z_cmd_start(int argc, char *argv[])
 {
-  printf("Starting Zorp Firewall Suite: ");
-  z_process_args("start", argc-2, &argv[2], z_process_start_instance, NULL, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Starting Zorp Firewall Suite: ",
+    "start", argc-2, &argv[2], z_process_start_instance, NULL, 1);
 }
 
 static int
 z_cmd_force_start(int argc, char *argv[])
 {
-  printf("Starting Zorp Firewall Suite: ");
-  z_process_args("start", argc-2, &argv[2], z_process_force_start_instance, NULL, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Starting Zorp Firewall Suite: ",
+    "start", argc-2, &argv[2], z_process_force_start_instance, NULL, 1);
 }
 
 static int
 z_cmd_stop(int argc, char *argv[])
 {
-  printf("Stopping Zorp Firewall Suite: ");
-  z_process_args("stop", argc-2, &argv[2], z_process_stop_instance, (void *) SIGTERM, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Stopping Zorp Firewall Suite: ",
+    "stop", argc-2, &argv[2], z_process_stop_instance, (void *) SIGTERM, 1);
 }
 
 static int
 z_cmd_force_stop(int argc, char *argv[])
 {
-  printf("Stopping Zorp Firewall Suite: ");
-  z_process_args("stop", argc-2, &argv[2], z_process_stop_instance, (void *) SIGKILL, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Stopping Zorp Firewall Suite: ",
+    "stop", argc-2, &argv[2], z_process_stop_instance, (void *) SIGKILL, 1);
 }
 
 static int
 z_cmd_restart(int argc, char *argv[])
 {
-  printf("Restarting Zorp Firewall Suite: ");
-  z_process_args("restart", argc-2, &argv[2], z_process_restart_instance, (void *) SIGTERM, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Restarting Zorp Firewall Suite: ",
+    "restart", argc-2, &argv[2], z_process_restart_instance, (void *) SIGTERM, 1);
 }
 
 static int
 z_cmd_force_restart(int argc, char *argv[])
 {
-  printf("Restarting Zorp Firewall Suite: ");
-  z_process_args("restart", argc-2, &argv[2], z_process_restart_instance, (void *) SIGKILL, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Restarting Zorp Firewall Suite: ",
+    "restart", argc-2, &argv[2], z_process_restart_instance, (void *) SIGKILL, 1);
 }
 
 static int
 z_cmd_reload(int argc, char *argv[])
 {
-  printf("Reloading Zorp Firewall Suite: ");
-  z_process_args("reload", argc-2, &argv[2], z_process_reload, NULL, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Reloading Zorp Firewall Suite: ",
+    "reload", argc-2, &argv[2], z_process_reload, NULL, 1);
 }
 
 static int
 z_cmd_reload_or_restart(int argc, char *argv[])
 {
-  printf("Reloading or Restarting Zorp Firewall Suite: ");
-  z_process_args("reload-or-restart", argc-2, &argv[2], z_process_reload_or_restart, NULL, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Reloading or Restarting Zorp Firewall Suite: ",
+    "reload-or-restart", argc-2, &argv[2], z_process_reload_or_restart, NULL, 1);
 }
 
 static int
 z_cmd_coredump(int argc, char *argv[])
 {
-  printf("Creating Zorp core dumps: ");
-  z_process_args("coredump", argc - 2, &argv[2], z_process_coredump, NULL, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Creating Zorp core dumps: ",
+    "coredump", argc - 2, &argv[2], z_process_coredump, NULL, 1);
 }
 
 static int
@@ -1757,18 +1901,96 @@ z_cmd_status(int argc, char *argv[])
       status_flags = ZORPCTL_STATUS_VERBOSE;
       processed_args++;
     }
-  z_process_args("status", argc - processed_args, &argv[processed_args], z_process_status_instance, &status_flags, 0);
-  return 0;
+  return z_process_args("status", argc - processed_args, &argv[processed_args], z_process_status_instance, &status_flags, 0);
 }
 
 static int
-z_cmd_gui_status(int argc, char *argv[])
+z_cmd_authorize(int argc, char *argv[])
 {
-  printf("\"instance\";\"status\";\"pid\";\"running threads\";\"total threads\";\"thread1avg\";\"thread5avg\";\"thread15avg\"\n");
-  z_process_args("gui-status", argc-2, &argv[2], z_process_gui_status_instance, NULL, 0);
-  return 0;
+  int res = 0;
+  GOptionContext *ctx;
+  GError *error = NULL;
+
+  gchar *auth_accept_sid  = NULL;
+  gchar *auth_reject_sid  = NULL;
+  gchar *auth_instance    = NULL;
+  gchar *auth_description = NULL;
+  gboolean auth_list_pending    = FALSE;
+  gboolean auth_list_csv        = FALSE;
+
+  GOptionEntry authorize_options[] =
+  {
+    { "list-pending", 'l',  0, G_OPTION_ARG_NONE,   &auth_list_pending, "Lists sessions for pending authorization",       NULL },
+    { "csv",          'c',  0, G_OPTION_ARG_NONE,   &auth_list_csv,     "Generates output of list-pending in csv format", NULL },
+    { "accept",       'a',  0, G_OPTION_ARG_STRING, &auth_accept_sid,   "Accepts authorization request",                  "<session id>" },
+    { "reject",       'r',  0, G_OPTION_ARG_STRING, &auth_reject_sid,   "Rejects authorization request",                  "<session id>" },
+    { "description",  'd',  0, G_OPTION_ARG_STRING, &auth_description,  "Description of accept/reject for logging ",      "<description>" },
+    { "instance",     'i',  0, G_OPTION_ARG_STRING, &auth_instance,     "Proxy instance which the session belongs to",    "<instance name>" },
+    { NULL,            0,   0, 0,                   0,                  NULL,                                             NULL },
+  };
+
+  --argc;
+  ++argv;
+  ctx = g_option_context_new(NULL);
+  g_option_context_add_main_entries(ctx, authorize_options, NULL);
+  if (!g_option_context_parse(ctx, &argc, &argv, &error))
+    {
+      fprintf(stderr, "zorpctl authorize: %s\n", error ? error->message : "Unknown error");
+      return 1;
+    }
+  g_option_context_free(ctx);
+
+  if (auth_list_csv)
+    auth_list_pending = TRUE;
+
+  if (auth_accept_sid || auth_reject_sid)
+    {
+      if (auth_list_pending)
+        {
+          fprintf(stderr, "zorpctl authorize: -l and -c cannot be used with -a or -c\n");
+          return 1;
+        }
+      if (!auth_instance)
+        {
+          fprintf(stderr, "zorpctl authorize: proxy instance is not specified\n");
+          return 1;          
+        }
+      if (!auth_description)
+        {
+          fprintf(stderr, "zorpctl authorize: description is not specified\n");
+          return 1;          
+        }
+    } /* if list-pending is used, description and instance parameters are ignored */
+  else
+    auth_list_pending = TRUE;
+
+  if (auth_accept_sid && auth_reject_sid)
+    {
+      fprintf(stderr, "zorpctl authorize: both reject and accept cannot be used\n");
+      return 1;          
+    }
+
+  if (auth_list_pending)
+    {
+      if (auth_instance)
+        {
+          char *procargv[2] = { auth_instance, NULL };
+          res = z_process_args("authorize_list",  1, procargv, z_process_authorize_list, (void *)&auth_list_csv, 0);
+        }
+      else
+        res = z_process_args("authorize_list",  0, 0, z_process_authorize_list, (void *)&auth_list_csv, 0);
+    }
+  else
+    {
+      char *procargv[2] = { auth_instance, NULL };
+      char *params[3] = { auth_accept_sid, auth_reject_sid, auth_description };
+      res = z_process_args("authorize",  1, procargv,  z_process_authorize, (void *)params, 0);
+    }
+
+  return res;
 }
 
+
 static int
 z_cmd_version(int argc UNUSED, char *argv[] UNUSED)
 {
@@ -1780,19 +2002,15 @@ z_cmd_version(int argc UNUSED, char *argv[] UNUSED)
 static int
 z_cmd_inclog(int argc, char *argv[])
 {
-  printf("Raising Zorp loglevel: ");
-  z_process_args("inclog", argc-2, &argv[2], z_process_signal_instance, (void *) SIGUSR1, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Raising Zorp loglevel: ",
+    "inclog", argc-2, &argv[2], z_process_signal_instance, (void *) SIGUSR1, 1);
 }
 
 static int
 z_cmd_declog(int argc, char *argv[])
 {
-  printf("Lowering Zorp loglevel: ");
-  z_process_args("declog", argc-2, &argv[2], z_process_signal_instance, (void *) SIGUSR2, 1);
-  printf("\n");
-  return 0;
+  return z_process_args_withmsg("Lowering Zorp loglevel: ",
+      "declog", argc-2, &argv[2], z_process_signal_instance, (void *) SIGUSR2, 1);
 }
 
 static int 
@@ -1839,7 +2057,7 @@ z_cmd_log(int argc, char *argv[])
   else if (logspec)
     res = z_process_args("log", argc, argv, z_process_log_logspec_instance, (char *) logspec, 1);
   else
-    z_process_args("log", argc, argv, z_process_log_status_instance, NULL, 0);
+    res = z_process_args("log", argc, argv, z_process_log_status_instance, NULL, 0);
 
   if (inc_verbosity || dec_verbosity || verbose_set || logspec)
     printf("\n");
@@ -1884,7 +2102,7 @@ z_cmd_deadlockcheck(int argc, char *argv[])
   else if (disable_check)
     res = z_process_args("deadlockcheck", argc, argv, z_process_deadlockcheck_disable_instance, NULL, 1);
   else
-    z_process_args("deadlockcheck", argc, argv, z_process_deadlockcheck_status_instance, NULL, 0);
+    res = z_process_args("deadlockcheck", argc, argv, z_process_deadlockcheck_status_instance, NULL, 0);
 
   if (enable_check || disable_check)
     printf("\n");
@@ -1928,9 +2146,16 @@ z_cmd_szig(int argc, char *argv[])
 
   if (!res)
     res = z_process_args("szig", argc, argv, z_process_kzorp_entries, 0, 0);
+
   return res;
 }
 
+static int
+z_cmd_conns(int argc, char *argv[])
+{
+  return z_process_args("conns", argc-2, &argv[2], z_process_conns_walk_instance, NULL, 0);
+}
+
 static int 
 z_cmd_usage(int argc UNUSED, char *argv[] UNUSED)
 {
@@ -1958,13 +2183,14 @@ ZCommand commands[] =
   { "coredump", z_cmd_coredump,    "Create core dumps of the specified Zorp instance(s)" },
   { "status", z_cmd_status,        "Status of the specified Zorp instance(s).\n"
                                    "\t\t    For additional information use status -v or --verbose option" },
-  { "gui-status", z_cmd_gui_status, "Status of the specified Zorp instance(s)" },
+  { "authorize", z_cmd_authorize,  "Lists and manages authorizations" },
   { "version", z_cmd_version,      "Display Zorp version information" },
   { "inclog",  z_cmd_inclog,       "Raise the specified Zorp instance(s) log level by one" },
   { "declog",  z_cmd_declog,       "Lower the specified Zorp instance(s) log level by one" },
   { "log",  z_cmd_log,             "Change and query Zorp log settings" },
   { "deadlockcheck",  z_cmd_deadlockcheck,             "Change and query Zorp deadlock checking settings" },
   { "szig",  z_cmd_szig,           "Display internal information from the given Zorp instance(s)" },
+  { "conns", z_cmd_conns,          "List the active connections" },
   { "help",  z_cmd_usage,          "Display this screen" },
   { NULL, NULL, NULL }
 };
@@ -1975,11 +2201,12 @@ main(int argc, char *argv[])
   int i;
   ZCommand *cmd = NULL;
   int rc = 0;
+  int config_ok = 1;
 
   setvbuf(stdout, NULL, _IONBF, 0);
   z_setup_signals();
-  z_parse_config();
-  z_parse_instances();
+  config_ok &= !!z_parse_config();
+  config_ok &= !!z_parse_instances();
   
   z_check_pidfile_dir();
   if (!z_check_config_dir())
@@ -2013,5 +2240,5 @@ main(int argc, char *argv[])
     }
   
   z_dump_errors();
-  return rc;
+  return rc ? rc : !config_ok;
 }
